import pandas as pd
import numpy as np
import torch
device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")

def get_hyperedges():
    graph_path = "./data/adj_text_format/HyperGraphPEMSBAY.txt"
    graph = pd.read_table(graph_path)
    graph_array = graph.columns.tolist()
    list_graph = graph_array[0]
    split_s = list_graph.split("], ")
    hyperedges = []
    for i in split_s:
        str_i = i.replace("[", "")
        str_final = str_i.replace("]", "")
        np_i = np.fromstring(str_final, dtype=int, sep=', ')
        hyperedges.append(np_i.tolist())
    return hyperedges


def construct_pemsbay_hyper_graph_laplacian(vertex_size=325):
    hyper_edges = get_hyperedges()
    correlation_matrix_H = np.zeros((vertex_size, len(hyper_edges)))
    vertex_degree_matrix_D = np.zeros(vertex_size)
    edge_degree_matrix_D = np.zeros(len(hyper_edges))
    hyper_edges_weight_matrix_W = np.ones(len(hyper_edges))  # 这里先默认所有的超边权重均为1
    identity_matrix_I = np.ones(vertex_size)
    for j in range(len(hyper_edges)):
        edge_degree_matrix_D[j] = len(hyper_edges[j])
    for node in range(vertex_size):
        for i in range(len(hyper_edges)):
            hyper_edge = hyper_edges[i]
            if node in hyper_edge:
                vertex_degree_matrix_D[node] += 1
                correlation_matrix_H[node][i] = 1
    H = np.mat(correlation_matrix_H)
    D_v_2 = np.mat(np.diag(np.power(vertex_degree_matrix_D, -0.5)))
    D_inv = np.mat(np.diag(np.power(edge_degree_matrix_D, -1)))
    W = np.mat(np.diag(hyper_edges_weight_matrix_W))
    I_m = np.mat(np.diag(identity_matrix_I))
    theta = D_v_2 * H * W * D_inv * H.transpose() * D_v_2
    Nor_hyper_graph_L = I_m - theta
    return torch.Tensor(Nor_hyper_graph_L).to(device)


def construct_pemsbay_line_graph_from_hyper_graph():
    hyper_edges = get_hyperedges()
    weight_matrix_line_graph = np.zeros((len(hyper_edges), len(hyper_edges)))
    for i in range(len(hyper_edges)):
        temp_i_hyper_edge = hyper_edges[i]
        for j in range(len(hyper_edges)):
            # 取出超边的交集与并集部分，计算线图的其权重,如果是本身，则权重为1，因为交集和并集都是自己
            inter_section = list(set(temp_i_hyper_edge).intersection(hyper_edges[j]))
            union_section = list(set(temp_i_hyper_edge).union(hyper_edges[j]))
            if inter_section:
                weight_matrix_line_graph[i][j] = len(inter_section) / len(union_section)
    D_hat = np.sum(weight_matrix_line_graph, 1)
    A_hat = np.mat(weight_matrix_line_graph)
    D_hat_2 = np.mat(np.diag(np.power(D_hat, -0.5)))
    line_graph_nor = D_hat_2 * A_hat * D_hat_2
    return torch.Tensor(line_graph_nor).to(device)

print(construct_pemsbay_line_graph_from_hyper_graph())
print(construct_pemsbay_hyper_graph_laplacian())