from dual_hypergraph_convolution import DualChannelGraphConv, LGraphConv, HGraphConv, GraphConv
from transformer_encoder import SeqTransformerModel
from convlstm import MyConvLSTM
import torch.nn as nn


class HyperGraphTransformerWeight(nn.Module):
    def __init__(self, input_feature, embedding_size, conv_output_size, n_node, hyper_graph_lap, line_graph_lap,
                 output_size,drop):  # hidden_size为节点的倍数
        super(HyperGraphTransformerWeight, self).__init__()
        self.num_node = n_node
        self.SBlock = DualChannelGraphConv(input_feature, embedding_size, conv_output_size,
                                           n_node, hyper_graph_lap, line_graph_lap,drop)
        self.decoder = MyConvLSTM(input_size=conv_output_size, hidden_size=64, f_dim=1, output_size=output_size,
                                  kernel_size=3, padding=1)

    def forward(self, x_input):
        embedding = self.SBlock(x_input)
        embedding_input = embedding.permute(0, 1, 3, 2)
        output = self.decoder(embedding_input)
        return output.squeeze(), 0

class HGraphTransformerWeight(nn.Module):
    def __init__(self, input_feature, embedding_size, conv_output_size, n_node, hyper_graph_lap,
                 output_size):  # hidden_size为节点的倍数
        super(HGraphTransformerWeight, self).__init__()
        self.num_node = n_node
        self.SBlock = HGraphConv(input_feature, embedding_size, conv_output_size,
                                           hyper_graph_lap)
        self.decoder = MyConvLSTM(input_size=conv_output_size, hidden_size=64, f_dim=1, output_size=output_size,
                                  kernel_size=3, padding=1)

    def forward(self, x_input):
        embedding = self.SBlock(x_input)
        embedding_input = embedding.permute(0, 1, 3, 2)
        output = self.decoder(embedding_input)
        return output.squeeze(), 0

class LGraphTransformerWeight(nn.Module):
    def __init__(self, input_feature, embedding_size, conv_output_size, n_node, line_graph_lap,
                 output_size):  # hidden_size为节点的倍数
        super(LGraphTransformerWeight, self).__init__()
        self.num_node = n_node
        self.SBlock = LGraphConv(input_feature, embedding_size, conv_output_size,
                                           n_node, line_graph_lap)
        self.decoder = MyConvLSTM(input_size=conv_output_size, hidden_size=64, f_dim=1, output_size=output_size,
                                  kernel_size=3, padding=1)

    def forward(self, x_input):
        embedding = self.SBlock(x_input)
        embedding_input = embedding.permute(0, 1, 3, 2)
        output = self.decoder(embedding_input)
        return output.squeeze(), 0


class GraphTransformerWeight(nn.Module):
    def __init__(self, input_feature, embedding_size, conv_output_size, n_node, graph_lap,
                 output_size):  # hidden_size为节点的倍数
        super(GraphTransformerWeight, self).__init__()
        self.num_node = n_node
        self.SBlock = GraphConv(input_feature, embedding_size, conv_output_size,
                                           graph_lap)
        self.decoder = MyConvLSTM(input_size=conv_output_size, hidden_size=64, f_dim=1, output_size=output_size,
                                  kernel_size=3, padding=1)

    def forward(self, x_input):
        embedding = self.SBlock(x_input)
        embedding_input = embedding.permute(0, 1, 3, 2)
        output = self.decoder(embedding_input)
        return output.squeeze(), 0


class HyperGraphTransformerEncoder(nn.Module):
    def __init__(self, input_feature, embedding_size, conv_output_size, n_node, hyper_graph_lap, line_graph_lap,
                 output_size,hidden_size_enc, filter_size, dropout_rate, n_layers):  # hidden_size为节点的倍数
        super(HyperGraphTransformerEncoder, self).__init__()
        self.num_node = n_node
        self.SBlock = DualChannelGraphConv(input_feature, embedding_size, conv_output_size,
                                           n_node, hyper_graph_lap, line_graph_lap, dropout_rate)
        self.encoder = SeqTransformerModel(hidden_size_enc, filter_size, dropout_rate, n_layers)
        self.decoder = MyConvLSTM(input_size=1, hidden_size=64, f_dim=conv_output_size, output_size=output_size,
                                  kernel_size=5, padding=1)

    def forward(self, x_input):
        x_output = self.SBlock(x_input)
        x_output = x_output.reshape(x_output.shape[0], x_output.shape[1], -1)
        embedding = self.encoder(x_output)
        embedding = embedding.reshape(embedding.shape[0], embedding.shape[1], self.num_node, -1)
        embedding_input = embedding.permute(0, 1, 3, 2)
        output = self.decoder(embedding_input)
        return output.squeeze(), 0
