import random
import torch
import torch.nn as nn
import numpy as np
from sklearn.preprocessing import StandardScaler
from model_hypergraph_lstm import HyperGraphTransformerWeight
from utils2 import construct_pemsbay_hyper_graph_laplacian, construct_pemsbay_line_graph_from_hyper_graph
from utils import load_data, data_transform, evaluate_model_hyper_graph_transformer_weight, evaluate_metric_hyper_graph_transformer
from torch.utils.data import DataLoader, TensorDataset
torch.manual_seed(2333)
torch.cuda.manual_seed(2333)
np.random.seed(2333)
random.seed(2333)
torch.backends.cudnn.deterministic = True
device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
data_path = "./data/pemsbay/Speed_325.csv"
save_path_raw = "./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster"
day_slot = 288
n_train, n_val, n_test = 35, 5, 5
batch_size = 50
epochs = 50
lr = 1e-3
n_his = 12
n_pred = 3
n_cluster = 64
output_dim = 8
input_feature = 1
embedding_size = 64
conv_output = 8
n_node = 325
hidden_size = conv_output * n_node
filter_size = 512
dropout_rate = 0
n_layers = 6
target_len = 3
# hyper_graph_lap_np = np.load("./data/adj_text_format/hyper_graph_lap_pemsbay.npy")
# hyper_graph_lap = torch.Tensor(hyper_graph_lap_np).to(device)
# line_graph_lap_np = np.load("./data/adj_text_format/line_graph_lap_pemsbay.npy")
# line_graph_lap = torch.Tensor(line_graph_lap_np).to(device)
clusters = [4, 8, 16, 32, 64, 128, 256]
for n_cluster in clusters:
    save_path = save_path_raw + str(n_cluster) + ".pt"
    print(save_path)
    hyper_graph_lap = construct_pemsbay_hyper_graph_laplacian(n_clusters=n_cluster)
    line_graph_lap = construct_pemsbay_line_graph_from_hyper_graph(n_clusters=n_cluster)
    train, val, test = load_data(data_path, n_train * day_slot, n_val * day_slot)  # 2017年3月6日到4月30日 ,除去周末，40天
    scaler = StandardScaler()
    train = scaler.fit_transform(train)
    val = scaler.transform(val)
    test = scaler.transform(test)
    x_train, y_train = data_transform(train, n_his, n_pred, day_slot, device)
    x_val, y_val = data_transform(val, n_his, n_pred, day_slot, device)
    x_test, y_test = data_transform(test, n_his, n_pred, day_slot, device)
    train_data = TensorDataset(x_train, y_train)
    train_iter = DataLoader(train_data, batch_size, shuffle=True)
    val_data = TensorDataset(x_val, y_val)
    val_iter = DataLoader(val_data, batch_size)
    test_data = TensorDataset(x_test, y_test)
    test_iter = DataLoader(test_data, batch_size)

    loss = nn.MSELoss()
    best_model = HyperGraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
                                  n_node=n_node, hyper_graph_lap=hyper_graph_lap, line_graph_lap=line_graph_lap,
                                 output_size=target_len,drop=dropout_rate).to(device)
    best_model.load_state_dict(torch.load(save_path))
    MAE, MAPE,RMSE = evaluate_metric_hyper_graph_transformer(best_model, test_iter, scaler)
    print("HyperGraph_pemsbay_15min")
    print("test metric:", "\nMAE:", MAE,"\nMAPE:", MAPE, ", RMSE:", RMSE)
    # model = HyperGraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
    #                                     n_node=n_node, hyper_graph_lap=hyper_graph_lap, line_graph_lap=line_graph_lap,
    #                                     output_size=target_len,drop=dropout_rate).to(device)
    # optimizer = torch.optim.Adam(model.parameters(), lr=lr)
    # scheduler = torch.optim.lr_scheduler.StepLR(optimizer, step_size=5, gamma=0.7)
    # min_val_loss = np.inf
    # for epoch in range(1, epochs + 1):
    #     l_sum, n = 0.0, 0
    #     model.train()
    #     for x, y in train_iter:
    #         #print(x.shape)
    #         model_outputs, enc_weight_output = model(x)
    #         final_weight = enc_weight_output
    #         l_loss = loss(model_outputs, y)
    #         optimizer.zero_grad()
    #         l_loss.backward()
    #         optimizer.step()
    #         l_sum += l_loss.item() * y.shape[0]
    #         n += y.shape[0]
    #     scheduler.step()
    #     val_loss = evaluate_model_hyper_graph_transformer_weight(model, loss, val_iter)
    #     if val_loss < min_val_loss:
    #         min_val_loss = val_loss
    #         torch.save(model.state_dict(), save_path)
    #     print("epoch", epoch, ", train loss:", l_sum / n, ", validation loss:", val_loss)


# best_model = HyperGraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
#                               n_node=n_node, hyper_graph_lap=hyper_graph_lap, line_graph_lap=line_graph_lap,
#                              output_size=target_len).to(device)
# best_model.load_state_dict(torch.load(save_path))
# MAE, MAPE,RMSE = evaluate_metric_hyper_graph_transformer(best_model, test_iter, scaler)
# print("HyperGraph_pemsbay_15min")
# print("test metric:", "\nMAE:", MAE,"\nMAPE:", MAPE, ", RMSE:", RMSE)
'''./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster4.pt
epoch 1 , train loss: 0.47660792501638116 , validation loss: 0.4033455031246814
epoch 2 , train loss: 0.2823524628874931 , validation loss: 0.3286378488346236
epoch 3 , train loss: 0.20824711714978233 , validation loss: 0.22141817347882767
epoch 4 , train loss: 0.15395257265813414 , validation loss: 0.1737942434854666
epoch 5 , train loss: 0.12959056937063612 , validation loss: 0.14978212732475263
epoch 6 , train loss: 0.11866472150323046 , validation loss: 0.13409590258492843
epoch 7 , train loss: 0.11288827944489613 , validation loss: 0.13027594191614963
epoch 8 , train loss: 0.10931112699113837 , validation loss: 0.12663905452197416
epoch 9 , train loss: 0.10672836998140166 , validation loss: 0.12221043648626698
epoch 10 , train loss: 0.10497839768646319 , validation loss: 0.12099371301136554
epoch 11 , train loss: 0.10371217064242128 , validation loss: 0.1209626039139643
epoch 12 , train loss: 0.10326059277156605 , validation loss: 0.12043705511340219
epoch 13 , train loss: 0.10295921016625544 , validation loss: 0.11931675607580533
epoch 14 , train loss: 0.10235290083013987 , validation loss: 0.11849058638326905
epoch 15 , train loss: 0.10211197755905463 , validation loss: 0.11794605822330992
epoch 16 , train loss: 0.10142734647760115 , validation loss: 0.11743864142969386
epoch 17 , train loss: 0.10122795944970683 , validation loss: 0.11693158283370819
epoch 18 , train loss: 0.10098161879956853 , validation loss: 0.1172487708440293
epoch 19 , train loss: 0.10075809067205735 , validation loss: 0.11633317324652123
epoch 20 , train loss: 0.10055438130405635 , validation loss: 0.11715305863055092
epoch 21 , train loss: 0.10019159602856421 , validation loss: 0.1164263738223181
epoch 22 , train loss: 0.10014280992406707 , validation loss: 0.11607790510674844
epoch 23 , train loss: 0.09988881097301855 , validation loss: 0.11622573626264904
epoch 24 , train loss: 0.09985616113519756 , validation loss: 0.11550959799290987
epoch 25 , train loss: 0.0997071377496945 , validation loss: 0.11552177775158354
epoch 26 , train loss: 0.09935166326816326 , validation loss: 0.11510971471706079
epoch 27 , train loss: 0.09915851943839192 , validation loss: 0.1151951832897109
epoch 28 , train loss: 0.099136865008928 , validation loss: 0.11488998550686666
epoch 29 , train loss: 0.09902502729134106 , validation loss: 0.11494932762084215
epoch 30 , train loss: 0.09891857795184915 , validation loss: 0.11475196175590041
epoch 31 , train loss: 0.09877916663449368 , validation loss: 0.11476865146976047
epoch 32 , train loss: 0.09866392020891712 , validation loss: 0.11442670039965964
epoch 33 , train loss: 0.0985955681924374 , validation loss: 0.11498059108149615
epoch 34 , train loss: 0.09855116683348117 , validation loss: 0.11447768419519792
epoch 35 , train loss: 0.09851885942977652 , validation loss: 0.11484423700938752
epoch 36 , train loss: 0.09828556489316909 , validation loss: 0.11433423823712967
epoch 37 , train loss: 0.0982628559863295 , validation loss: 0.11420093871701759
epoch 38 , train loss: 0.09822035648106739 , validation loss: 0.11450923024134665
epoch 39 , train loss: 0.09814219218078875 , validation loss: 0.11401954372952716
epoch 40 , train loss: 0.0981269858664522 , validation loss: 0.11413483974849703
epoch 41 , train loss: 0.09800768100108098 , validation loss: 0.11387130761375645
epoch 42 , train loss: 0.09795711504240556 , validation loss: 0.11390259738327009
epoch 43 , train loss: 0.0979147375454548 , validation loss: 0.11378852477007456
epoch 44 , train loss: 0.09790442457030116 , validation loss: 0.11386756118627557
epoch 45 , train loss: 0.09789935132587109 , validation loss: 0.11378526766703174
epoch 46 , train loss: 0.09784087524198155 , validation loss: 0.11378808054886921
epoch 47 , train loss: 0.0977807845300755 , validation loss: 0.11378805078261678
epoch 48 , train loss: 0.09774189562102382 , validation loss: 0.11360414826732526
epoch 49 , train loss: 0.09771023869691443 , validation loss: 0.11361543022442429
epoch 50 , train loss: 0.09768736035564747 , validation loss: 0.11368445086956253
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster8.pt
epoch 1 , train loss: 0.4597432165417964 , validation loss: 0.37681322499269926
epoch 2 , train loss: 0.25127823923981973 , validation loss: 0.2347477135584006
epoch 3 , train loss: 0.16441184463785707 , validation loss: 0.16839840235713885
epoch 4 , train loss: 0.13304256945242915 , validation loss: 0.1463798409589895
epoch 5 , train loss: 0.11972562780005047 , validation loss: 0.1326391167926371
epoch 6 , train loss: 0.11274467399631366 , validation loss: 0.12647513928587892
epoch 7 , train loss: 0.10917856142046138 , validation loss: 0.12443662020795132
epoch 8 , train loss: 0.10737805603257103 , validation loss: 0.12344826701965542
epoch 9 , train loss: 0.10614087287486332 , validation loss: 0.12316575654971902
epoch 10 , train loss: 0.10567174677988153 , validation loss: 0.12083921263071767
epoch 11 , train loss: 0.10444658447248362 , validation loss: 0.11964492574710169
epoch 12 , train loss: 0.10406121071953114 , validation loss: 0.11952609830051361
epoch 13 , train loss: 0.1037681223378926 , validation loss: 0.11904606405262066
epoch 14 , train loss: 0.10332322855467364 , validation loss: 0.1207520793063292
epoch 15 , train loss: 0.10297730111730405 , validation loss: 0.11848200924319632
epoch 16 , train loss: 0.10217484234411353 , validation loss: 0.11930777170157832
epoch 17 , train loss: 0.10184070197452814 , validation loss: 0.11875406598191414
epoch 18 , train loss: 0.10158635861547748 , validation loss: 0.1170551446140575
epoch 19 , train loss: 0.10115708012188157 , validation loss: 0.11687163419193328
epoch 20 , train loss: 0.10091143088013782 , validation loss: 0.1171255703853788
epoch 21 , train loss: 0.10039904943177183 , validation loss: 0.1161380300257114
epoch 22 , train loss: 0.10025668030623373 , validation loss: 0.11593567219564743
epoch 23 , train loss: 0.10006365787552418 , validation loss: 0.11607226686451645
epoch 24 , train loss: 0.09995297343671478 , validation loss: 0.11573742056040824
epoch 25 , train loss: 0.0997859001380106 , validation loss: 0.11630963823053546
epoch 26 , train loss: 0.09953888404811294 , validation loss: 0.11533374349025967
epoch 27 , train loss: 0.09937295425461606 , validation loss: 0.11503249290364007
epoch 28 , train loss: 0.0993193905936555 , validation loss: 0.11549995248885651
epoch 29 , train loss: 0.09911858101266541 , validation loss: 0.1149003637240273
epoch 30 , train loss: 0.09904146353423747 , validation loss: 0.11507051888886265
epoch 31 , train loss: 0.09884973384015674 , validation loss: 0.11472008816353665
epoch 32 , train loss: 0.09870722303803096 , validation loss: 0.114861213380065
epoch 33 , train loss: 0.09867118729589894 , validation loss: 0.11502076458967175
epoch 34 , train loss: 0.09859111702496073 , validation loss: 0.11442723295632853
epoch 35 , train loss: 0.09848680057712184 , validation loss: 0.11469975327432456
epoch 36 , train loss: 0.09829885835128738 , validation loss: 0.1143266965677119
epoch 37 , train loss: 0.09827414810362012 , validation loss: 0.11433278601312027
epoch 38 , train loss: 0.09817302704637242 , validation loss: 0.11424345911796807
epoch 39 , train loss: 0.09810811206044621 , validation loss: 0.11420149051964923
epoch 40 , train loss: 0.09805343185638117 , validation loss: 0.11396960659571419
epoch 41 , train loss: 0.09794139914050032 , validation loss: 0.11397573580168543
epoch 42 , train loss: 0.09793293055285045 , validation loss: 0.11399881381836932
epoch 43 , train loss: 0.09785173780006831 , validation loss: 0.11400113578196375
epoch 44 , train loss: 0.09787643764587256 , validation loss: 0.11384014779749961
epoch 45 , train loss: 0.09779488904681452 , validation loss: 0.11392547465604262
epoch 46 , train loss: 0.09771198404217683 , validation loss: 0.11392438041180777
epoch 47 , train loss: 0.09766380728597857 , validation loss: 0.11383395134168996
epoch 48 , train loss: 0.09764450649977702 , validation loss: 0.11397791311052059
epoch 49 , train loss: 0.09762621080102774 , validation loss: 0.11377480676329293
epoch 50 , train loss: 0.09757684198787314 , validation loss: 0.11394184530187892
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster16.pt
epoch 1 , train loss: 0.4795434176892905 , validation loss: 0.4107604510654435
epoch 2 , train loss: 0.2512909688610997 , validation loss: 0.25188485572785696
epoch 3 , train loss: 0.175130480885518 , validation loss: 0.18207411879647245
epoch 4 , train loss: 0.14008889369880012 , validation loss: 0.15348185786849874
epoch 5 , train loss: 0.12502223399651133 , validation loss: 0.13964993210660304
epoch 6 , train loss: 0.11485765779956934 , validation loss: 0.12723382196966906
epoch 7 , train loss: 0.10881414540854034 , validation loss: 0.12359605558083675
epoch 8 , train loss: 0.10665962706946727 , validation loss: 0.12143576449717228
epoch 9 , train loss: 0.10514843476818468 , validation loss: 0.12009907777205632
epoch 10 , train loss: 0.10418541391483108 , validation loss: 0.11937209427492204
epoch 11 , train loss: 0.1029912075359947 , validation loss: 0.11824154657972898
epoch 12 , train loss: 0.1024998638015879 , validation loss: 0.11795119165933006
epoch 13 , train loss: 0.10204291820213862 , validation loss: 0.11767078429429673
epoch 14 , train loss: 0.10170810056925499 , validation loss: 0.11684222700180641
epoch 15 , train loss: 0.10122943837594417 , validation loss: 0.1163324658282867
epoch 16 , train loss: 0.10069241903812115 , validation loss: 0.11598830107348712
epoch 17 , train loss: 0.10035344118721137 , validation loss: 0.11582439637846871
epoch 18 , train loss: 0.10017292023157852 , validation loss: 0.11544268317249674
epoch 19 , train loss: 0.09985723104895872 , validation loss: 0.11531203182784902
epoch 20 , train loss: 0.09968364691078306 , validation loss: 0.11528877108870007
epoch 21 , train loss: 0.09925320230765818 , validation loss: 0.11483061775393848
epoch 22 , train loss: 0.09908056620169439 , validation loss: 0.11457777002571358
epoch 23 , train loss: 0.09888516500321491 , validation loss: 0.1143261055972939
epoch 24 , train loss: 0.09871613670234174 , validation loss: 0.11439303095447496
epoch 25 , train loss: 0.09869581640382961 , validation loss: 0.11391332917096861
epoch 26 , train loss: 0.09824430151996429 , validation loss: 0.11396914081454419
epoch 27 , train loss: 0.09813676349133142 , validation loss: 0.11386196524272815
epoch 28 , train loss: 0.09805821926842828 , validation loss: 0.11370639172083903
epoch 29 , train loss: 0.09794614378851181 , validation loss: 0.11345252175765358
epoch 30 , train loss: 0.09782723431716753 , validation loss: 0.11347609787500575
epoch 31 , train loss: 0.0976034685201008 , validation loss: 0.11315150154644663
epoch 32 , train loss: 0.09759015527914496 , validation loss: 0.1130903834480182
epoch 33 , train loss: 0.09749150865337954 , validation loss: 0.11322424693807773
epoch 34 , train loss: 0.09744896924420206 , validation loss: 0.11297286598071678
epoch 35 , train loss: 0.09727532122453325 , validation loss: 0.11289074727638279
epoch 36 , train loss: 0.097147631912812 , validation loss: 0.11288880196119684
epoch 37 , train loss: 0.09708646150278186 , validation loss: 0.11272616358235964
epoch 38 , train loss: 0.09697733540687158 , validation loss: 0.11280150313058554
epoch 39 , train loss: 0.09694641123355496 , validation loss: 0.11260260543121928
epoch 40 , train loss: 0.09692965558278355 , validation loss: 0.11253313873330255
epoch 41 , train loss: 0.09676427504499353 , validation loss: 0.11253973615266695
epoch 42 , train loss: 0.096726040329512 , validation loss: 0.11259302886471965
epoch 43 , train loss: 0.09670103258086792 , validation loss: 0.11241135507455242
epoch 44 , train loss: 0.0966348883573184 , validation loss: 0.1124182765342793
epoch 45 , train loss: 0.09660222627244464 , validation loss: 0.112419851957918
epoch 46 , train loss: 0.09653331362169575 , validation loss: 0.11231712651847385
epoch 47 , train loss: 0.09646930235835796 , validation loss: 0.11239066908283132
epoch 48 , train loss: 0.09647919328051509 , validation loss: 0.11219749520138456
epoch 49 , train loss: 0.09642764483292753 , validation loss: 0.11249903073175146
epoch 50 , train loss: 0.0964143954994804 , validation loss: 0.11221462965071606
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster32.pt
epoch 1 , train loss: 0.46110549676098755 , validation loss: 0.31127582470778836
epoch 2 , train loss: 0.16874099685770738 , validation loss: 0.1525944061538642
epoch 3 , train loss: 0.12652481889262307 , validation loss: 0.1393597531276826
epoch 4 , train loss: 0.11948237779709284 , validation loss: 0.13262576475572585
epoch 5 , train loss: 0.11298931725460079 , validation loss: 0.12609330711203062
epoch 6 , train loss: 0.10882960298074562 , validation loss: 0.12595571496330707
epoch 7 , train loss: 0.10657192152838273 , validation loss: 0.12087549715179778
epoch 8 , train loss: 0.10398460711762832 , validation loss: 0.11834431526976646
epoch 9 , train loss: 0.1021083396626478 , validation loss: 0.11679038800162052
epoch 10 , train loss: 0.10134048455178164 , validation loss: 0.1170468299943507
epoch 11 , train loss: 0.1007250785340482 , validation loss: 0.11582440213890864
epoch 12 , train loss: 0.10021361938047883 , validation loss: 0.11565245336490437
epoch 13 , train loss: 0.09986043953280842 , validation loss: 0.11467215596178922
epoch 14 , train loss: 0.09973604784933834 , validation loss: 0.11443759398579532
epoch 15 , train loss: 0.09941898304163492 , validation loss: 0.11478743644203578
epoch 16 , train loss: 0.09867408990011187 , validation loss: 0.11387326766527572
epoch 17 , train loss: 0.09859832179346915 , validation loss: 0.11352753556144664
epoch 18 , train loss: 0.09835769219185199 , validation loss: 0.11402720068720501
epoch 19 , train loss: 0.09825854810684873 , validation loss: 0.11385691406055894
epoch 20 , train loss: 0.09805037669566805 , validation loss: 0.11349246536433363
epoch 21 , train loss: 0.09771196678639538 , validation loss: 0.11282826223932253
epoch 22 , train loss: 0.09752865130530328 , validation loss: 0.11330081710714596
epoch 23 , train loss: 0.09739501036844679 , validation loss: 0.11281533841265425
epoch 24 , train loss: 0.09725270736748769 , validation loss: 0.1124947830886737
epoch 25 , train loss: 0.09710041492334441 , validation loss: 0.11264654000063479
epoch 26 , train loss: 0.09679117629227685 , validation loss: 0.11240156225039383
epoch 27 , train loss: 0.09674483501194502 , validation loss: 0.11218622243159299
epoch 28 , train loss: 0.09652394613813595 , validation loss: 0.11232761818559639
epoch 29 , train loss: 0.09645982269041085 , validation loss: 0.1123918046380572
epoch 30 , train loss: 0.09634722573115855 , validation loss: 0.11217951479698553
epoch 31 , train loss: 0.09616498041451034 , validation loss: 0.1120230327873003
epoch 32 , train loss: 0.09607196556465912 , validation loss: 0.11199245818673502
epoch 33 , train loss: 0.09604679271156086 , validation loss: 0.11214688873322881
epoch 34 , train loss: 0.09598147368582007 , validation loss: 0.11168181535626763
epoch 35 , train loss: 0.09584661452551699 , validation loss: 0.11196140867393455
epoch 36 , train loss: 0.09570562668800554 , validation loss: 0.11180650438553559
epoch 37 , train loss: 0.09558908453972254 , validation loss: 0.1118151031268582
epoch 38 , train loss: 0.09557540589220104 , validation loss: 0.11170368410736703
epoch 39 , train loss: 0.09553142321736896 , validation loss: 0.11175425547786239
epoch 40 , train loss: 0.09544706557459012 , validation loss: 0.11153688492616419
epoch 41 , train loss: 0.09531914247448155 , validation loss: 0.11169781297783919
epoch 42 , train loss: 0.09527900728628028 , validation loss: 0.11156635437534551
epoch 43 , train loss: 0.09524352347625067 , validation loss: 0.1117685676937198
epoch 44 , train loss: 0.09522497753679247 , validation loss: 0.11153277085643902
epoch 45 , train loss: 0.09515768509327427 , validation loss: 0.11148121978130006
epoch 46 , train loss: 0.09507222853646252 , validation loss: 0.11168407408827878
epoch 47 , train loss: 0.09504806582998758 , validation loss: 0.11147305719635381
epoch 48 , train loss: 0.09499612094828759 , validation loss: 0.11146845898484992
epoch 49 , train loss: 0.09500951689516648 , validation loss: 0.11151053346598255
epoch 50 , train loss: 0.09495523636134065 , validation loss: 0.11185041291872325
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster64.pt
epoch 1 , train loss: 0.4272920580962322 , validation loss: 0.22517326241702465
epoch 2 , train loss: 0.1470061308314928 , validation loss: 0.14714527792051788
epoch 3 , train loss: 0.11580152556627765 , validation loss: 0.1281310836287671
epoch 4 , train loss: 0.10749211994025878 , validation loss: 0.12182468789164008
epoch 5 , train loss: 0.10448166342743745 , validation loss: 0.12009572098829985
epoch 6 , train loss: 0.10271286446569705 , validation loss: 0.11745414173206314
epoch 7 , train loss: 0.10164320253068412 , validation loss: 0.11658359323949478
epoch 8 , train loss: 0.10103183232704716 , validation loss: 0.11914015256513495
epoch 9 , train loss: 0.10050073560542971 , validation loss: 0.11521994224768851
epoch 10 , train loss: 0.100096170987779 , validation loss: 0.1151381625170009
epoch 11 , train loss: 0.0992245714601636 , validation loss: 0.11419417293693523
epoch 12 , train loss: 0.09884332636638063 , validation loss: 0.11399720092712079
epoch 13 , train loss: 0.09853348829049673 , validation loss: 0.11423187271171022
epoch 14 , train loss: 0.09822728403456613 , validation loss: 0.11356700671313034
epoch 15 , train loss: 0.09813493653788409 , validation loss: 0.11317590853015234
epoch 16 , train loss: 0.09744625833116834 , validation loss: 0.11296664416106454
epoch 17 , train loss: 0.09724234530602743 , validation loss: 0.11336900396632846
epoch 18 , train loss: 0.09695963257525365 , validation loss: 0.11319538465229226
epoch 19 , train loss: 0.09677189086937273 , validation loss: 0.1125012042794051
epoch 20 , train loss: 0.0966713727051096 , validation loss: 0.11312463833803715
epoch 21 , train loss: 0.09624014531192239 , validation loss: 0.11257870686345431
epoch 22 , train loss: 0.0961044935115537 , validation loss: 0.11218639648173442
epoch 23 , train loss: 0.09586465551217781 , validation loss: 0.11191983934253408
epoch 24 , train loss: 0.09572463857090727 , validation loss: 0.11180116289210712
epoch 25 , train loss: 0.09564944349850782 , validation loss: 0.11171419133748221
epoch 26 , train loss: 0.09528828742877936 , validation loss: 0.11154633633785371
epoch 27 , train loss: 0.09514395824582884 , validation loss: 0.1116321564973163
epoch 28 , train loss: 0.09504711011070283 , validation loss: 0.11164837377474454
epoch 29 , train loss: 0.094904821838988 , validation loss: 0.1115090107507443
epoch 30 , train loss: 0.09482371450271847 , validation loss: 0.1120444070615856
epoch 31 , train loss: 0.09456044634994558 , validation loss: 0.11136187251266438
epoch 32 , train loss: 0.09443397713965505 , validation loss: 0.11131685260738673
epoch 33 , train loss: 0.09444774755670014 , validation loss: 0.11133633415490898
epoch 34 , train loss: 0.09429478904670427 , validation loss: 0.1113203801671836
epoch 35 , train loss: 0.0942633148420996 , validation loss: 0.1113248271646916
epoch 36 , train loss: 0.09408225700456573 , validation loss: 0.11150518884978375
epoch 37 , train loss: 0.09404219299203054 , validation loss: 0.11131505729697885
epoch 38 , train loss: 0.09393406450090278 , validation loss: 0.11115834968552608
epoch 39 , train loss: 0.09384435653012357 , validation loss: 0.1112178171474131
epoch 40 , train loss: 0.0938063222432605 , validation loss: 0.11107134395250093
epoch 41 , train loss: 0.09367592667724196 , validation loss: 0.11117150687598427
epoch 42 , train loss: 0.09363542604908665 , validation loss: 0.1109012799019647
epoch 43 , train loss: 0.09360386042545518 , validation loss: 0.11112726131640707
epoch 44 , train loss: 0.09359126768888378 , validation loss: 0.11098501762884046
epoch 45 , train loss: 0.09347571356583298 , validation loss: 0.11110527745273775
epoch 46 , train loss: 0.09341527943741285 , validation loss: 0.11109377948538988
epoch 47 , train loss: 0.09336530393985501 , validation loss: 0.11092915397332916
epoch 48 , train loss: 0.09335064324353008 , validation loss: 0.11125796810235461
epoch 49 , train loss: 0.0933127370788159 , validation loss: 0.11103358914496943
epoch 50 , train loss: 0.09326942723140441 , validation loss: 0.11093496663103618
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster128.pt
epoch 1 , train loss: 0.3608612776905147 , validation loss: 0.17936147189385263
epoch 2 , train loss: 0.12252974935436493 , validation loss: 0.12996718286674086
epoch 3 , train loss: 0.10917964619346206 , validation loss: 0.12551559677786336
epoch 4 , train loss: 0.10650053462818201 , validation loss: 0.12185233631148573
epoch 5 , train loss: 0.10453555518279892 , validation loss: 0.11897473340938726
epoch 6 , train loss: 0.10274091774996444 , validation loss: 0.11815548101348419
epoch 7 , train loss: 0.10225044947003652 , validation loss: 0.11675821361433672
epoch 8 , train loss: 0.10101818463510032 , validation loss: 0.11598461183939213
epoch 9 , train loss: 0.10062369492255528 , validation loss: 0.11541558726828585
epoch 10 , train loss: 0.10031156161846733 , validation loss: 0.1168505414805052
epoch 11 , train loss: 0.09929306048748099 , validation loss: 0.11442490991167355
epoch 12 , train loss: 0.09910964294922311 , validation loss: 0.11396030257887843
epoch 13 , train loss: 0.09879674097404605 , validation loss: 0.11336965115765137
epoch 14 , train loss: 0.09827243650278193 , validation loss: 0.11371641198119811
epoch 15 , train loss: 0.09811817680031025 , validation loss: 0.11386790550452941
epoch 16 , train loss: 0.09757050310434161 , validation loss: 0.11313148572069792
epoch 17 , train loss: 0.09744644335359692 , validation loss: 0.11254545395365431
epoch 18 , train loss: 0.09724905304892967 , validation loss: 0.1123302197224405
epoch 19 , train loss: 0.09706288131455794 , validation loss: 0.11258948095230477
epoch 20 , train loss: 0.09691380544330355 , validation loss: 0.1129884920494446
epoch 21 , train loss: 0.09651242534558427 , validation loss: 0.112288786822346
epoch 22 , train loss: 0.09634256979247592 , validation loss: 0.11208129018385926
epoch 23 , train loss: 0.09633460589229671 , validation loss: 0.11156029082029269
epoch 24 , train loss: 0.09605312075319053 , validation loss: 0.11135398814038393
epoch 25 , train loss: 0.09604887551061546 , validation loss: 0.11151402313371746
epoch 26 , train loss: 0.09560219640907354 , validation loss: 0.11138567962717294
epoch 27 , train loss: 0.0956009994913044 , validation loss: 0.11166597976919446
epoch 28 , train loss: 0.09545598736993813 , validation loss: 0.11136830117258824
epoch 29 , train loss: 0.09539949125341816 , validation loss: 0.11154281313410562
epoch 30 , train loss: 0.09527540177416541 , validation loss: 0.11119159066727495
epoch 31 , train loss: 0.09504115088952729 , validation loss: 0.11121508776955036
epoch 32 , train loss: 0.09492987516935615 , validation loss: 0.11084364082135165
epoch 33 , train loss: 0.0948376070793529 , validation loss: 0.1111362112819619
epoch 34 , train loss: 0.09481277893234334 , validation loss: 0.11132563837023841
epoch 35 , train loss: 0.09476472425230374 , validation loss: 0.11113336207831882
epoch 36 , train loss: 0.09458800064473903 , validation loss: 0.11074510462110876
epoch 37 , train loss: 0.09450203763005258 , validation loss: 0.11091700294893539
epoch 38 , train loss: 0.09445130697698091 , validation loss: 0.11075613088956236
epoch 39 , train loss: 0.09438311057866865 , validation loss: 0.11067444650862539
epoch 40 , train loss: 0.09434048844869855 , validation loss: 0.11092392561223924
epoch 41 , train loss: 0.09423363242778147 , validation loss: 0.1105764648886447
epoch 42 , train loss: 0.09421122020676587 , validation loss: 0.11060880176756621
epoch 43 , train loss: 0.094134293218177 , validation loss: 0.11063034127479382
epoch 44 , train loss: 0.09408690046559383 , validation loss: 0.1105198091280189
epoch 45 , train loss: 0.09406156455022488 , validation loss: 0.11066097418396077
epoch 46 , train loss: 0.09397966322453788 , validation loss: 0.1109071566159468
epoch 47 , train loss: 0.09395002613952148 , validation loss: 0.11052988766698046
epoch 48 , train loss: 0.09391264746443595 , validation loss: 0.11068411708813752
epoch 49 , train loss: 0.09387855974607781 , validation loss: 0.11060911868911183
epoch 50 , train loss: 0.09384195047318768 , validation loss: 0.11049838662298134
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster256.pt
epoch 1 , train loss: 0.26426954308731493 , validation loss: 0.13571112879617322
epoch 2 , train loss: 0.11156275687317219 , validation loss: 0.12476441796252064
epoch 3 , train loss: 0.1064474498648328 , validation loss: 0.12122474699261589
epoch 4 , train loss: 0.10447028734132975 , validation loss: 0.11935333521265071
epoch 5 , train loss: 0.10277151750281395 , validation loss: 0.11859704626736524
epoch 6 , train loss: 0.10149468308560988 , validation loss: 0.11655247167393251
epoch 7 , train loss: 0.10130648136729281 , validation loss: 0.1159163177519041
epoch 8 , train loss: 0.1003444421816087 , validation loss: 0.11592338888542854
epoch 9 , train loss: 0.10005901615627644 , validation loss: 0.11499302968970265
epoch 10 , train loss: 0.09935054330541425 , validation loss: 0.11527703845132288
epoch 11 , train loss: 0.09863713810147993 , validation loss: 0.1144094715475042
epoch 12 , train loss: 0.09821724133884707 , validation loss: 0.11359849224503067
epoch 13 , train loss: 0.09791979234049267 , validation loss: 0.11306970338649806
epoch 14 , train loss: 0.09757393347674007 , validation loss: 0.1137807181173959
epoch 15 , train loss: 0.09710981895906932 , validation loss: 0.11310825112917826
epoch 16 , train loss: 0.09665427398847867 , validation loss: 0.11206380431739114
epoch 17 , train loss: 0.09619306954451036 , validation loss: 0.11196185825476633
epoch 18 , train loss: 0.09605768006840051 , validation loss: 0.11199636672773436
epoch 19 , train loss: 0.09588259858160779 , validation loss: 0.11229210422047278
epoch 20 , train loss: 0.09558914712083735 , validation loss: 0.11145001829629825
epoch 21 , train loss: 0.09518043097391768 , validation loss: 0.11141665005642296
epoch 22 , train loss: 0.09497999979426372 , validation loss: 0.11084576684309227
epoch 23 , train loss: 0.09474640066752356 , validation loss: 0.11072018867601277
epoch 24 , train loss: 0.09455469785492165 , validation loss: 0.11120740011860253
epoch 25 , train loss: 0.09446177138809504 , validation loss: 0.11071266795088566
epoch 26 , train loss: 0.0939726380297 , validation loss: 0.11077831585988171
epoch 27 , train loss: 0.09389724492176847 , validation loss: 0.11141807986732276
epoch 28 , train loss: 0.0937268339126267 , validation loss: 0.11078163559360198
epoch 29 , train loss: 0.09375139918179298 , validation loss: 0.11071326042064139
epoch 30 , train loss: 0.09355264999392358 , validation loss: 0.11078696500661424
epoch 31 , train loss: 0.09328573362409526 , validation loss: 0.11061859972110517
epoch 32 , train loss: 0.09315903718771913 , validation loss: 0.11035258656416352
epoch 33 , train loss: 0.09306532995553156 , validation loss: 0.11057527277507732
epoch 34 , train loss: 0.09292094816795901 , validation loss: 0.11089141363652162
epoch 35 , train loss: 0.09283819410691002 , validation loss: 0.11050703008009916
epoch 36 , train loss: 0.0926457219203109 , validation loss: 0.1105633598134419
epoch 37 , train loss: 0.09257855848236857 , validation loss: 0.11023802692785357
epoch 38 , train loss: 0.09248705711805 , validation loss: 0.1106073323846333
epoch 39 , train loss: 0.0925404117145642 , validation loss: 0.11046471153582822
epoch 40 , train loss: 0.09241898759867828 , validation loss: 0.11048940764396246
epoch 41 , train loss: 0.09228996440505878 , validation loss: 0.11007934720138712
epoch 42 , train loss: 0.09221484055935468 , validation loss: 0.1103656221589808
epoch 43 , train loss: 0.09214426064224779 , validation loss: 0.1103832906532254
epoch 44 , train loss: 0.09211497103033528 , validation loss: 0.1105163058179624
epoch 45 , train loss: 0.092046691062488 , validation loss: 0.11018503515505403
epoch 46 , train loss: 0.09194630724503196 , validation loss: 0.11006256064300755
epoch 47 , train loss: 0.09190762731286593 , validation loss: 0.1101234212928176
epoch 48 , train loss: 0.09187084962408205 , validation loss: 0.11024858904723848
epoch 49 , train loss: 0.0918183912243919 , validation loss: 0.11041606129991346
epoch 50 , train loss: 0.09180288339211307 , validation loss: 0.11047249170612732
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster4.pt
HyperGraph_pemsbay_15min
test metric: 
MAE: 1.4200863608970504 
MAPE: 0.033515020707260236 , RMSE: 2.6511146679557345
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster8.pt
HyperGraph_pemsbay_15min
test metric: 
MAE: 1.4195527400063805 
MAPE: 0.03327700377136504 , RMSE: 2.623420103914667
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster16.pt
HyperGraph_pemsbay_15min
test metric: 
MAE: 1.407938213725897 
MAPE: 0.032762564244116506 , RMSE: 2.605066907845606
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster32.pt
HyperGraph_pemsbay_15min
test metric: 
MAE: 1.4140020173398542 
MAPE: 0.0329099238423059 , RMSE: 2.6001737349347573
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster64.pt
HyperGraph_pemsbay_15min
test metric: 
MAE: 1.4096817304529385 
MAPE: 0.03283832571871863 , RMSE: 2.5970129339426036
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster128.pt
HyperGraph_pemsbay_15min
test metric: 
MAE: 1.4034970399134448 
MAPE: 0.03230021593413984 , RMSE: 2.582946810101268
./pretrain_model/pemsbay/model_pemsbay_HyperGCN_cluster256.pt
HyperGraph_pemsbay_15min
test metric: 
MAE: 1.4031871417947968 
MAPE: 0.03262816808391316 , RMSE: 2.591341165926777'''