import random
import torch
import torch.nn as nn
import numpy as np
from sklearn.preprocessing import StandardScaler
from model_hypergraph_lstm import HyperGraphTransformerWeight
from utils2 import construct_pemsbay_hyper_graph_laplacian, construct_pemsbay_line_graph_from_hyper_graph
from utils import load_data, data_transform, evaluate_model_hyper_graph_transformer_weight, evaluate_metric_hyper_graph_transformer
from torch.utils.data import DataLoader, TensorDataset
torch.manual_seed(2333)
torch.cuda.manual_seed(2333)
np.random.seed(2333)
random.seed(2333)
torch.backends.cudnn.deterministic = True
device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
data_path = "./data/pemsbay/Speed_325.csv"
save_path = "./pretrain_model/pemsbay/model_pemsbay_HyperGCN_6_v1.pt"
day_slot = 288
n_train, n_val, n_test = 35, 5, 5
batch_size = 50
epochs = 50
lr = 1e-3
n_his = 12
n_pred = 4
n_cluster = 128
output_dim = 8
input_feature = 1
embedding_size = 64
conv_output = 8
n_node = 325
# hyper_graph_lap_np = np.load("./data/adj_text_format/hyper_graph_lap_pemsbay.npy")
# hyper_graph_lap = torch.Tensor(hyper_graph_lap_np).to(device)
# line_graph_lap_np = np.load("./data/adj_text_format/line_graph_lap_pemsbay.npy")
# line_graph_lap = torch.Tensor(line_graph_lap_np).to(device)
hyper_graph_lap = construct_pemsbay_hyper_graph_laplacian(n_clusters=n_cluster)
line_graph_lap = construct_pemsbay_line_graph_from_hyper_graph(n_clusters=n_cluster)
filter_size = 512
dropout_rate = 0
n_layers = 6
target_len = 4
train, val, test = load_data(data_path, n_train * day_slot, n_val * day_slot)  # 2017年3月6日到4月30日 ,除去周末，40天
scaler = StandardScaler()
train = scaler.fit_transform(train)
val = scaler.transform(val)
test = scaler.transform(test)
x_train, y_train = data_transform(train, n_his, n_pred, day_slot, device)
x_val, y_val = data_transform(val, n_his, n_pred, day_slot, device)
x_test, y_test = data_transform(test, n_his, n_pred, day_slot, device)
train_data = TensorDataset(x_train, y_train)
train_iter = DataLoader(train_data, batch_size, shuffle=True)
val_data = TensorDataset(x_val, y_val)
val_iter = DataLoader(val_data, batch_size)
test_data = TensorDataset(x_test, y_test)
test_iter = DataLoader(test_data, batch_size)

loss = nn.MSELoss()
# model = HyperGraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
#                                     n_node=n_node, hyper_graph_lap=hyper_graph_lap, line_graph_lap=line_graph_lap,
#                                     output_size=target_len,drop=dropout_rate).to(device)
# optimizer = torch.optim.Adam(model.parameters(), lr=lr)
# scheduler = torch.optim.lr_scheduler.StepLR(optimizer, step_size=5, gamma=0.7)
# min_val_loss = np.inf
# final_weight = 0
# for epoch in range(1, epochs + 1):
#     l_sum, n = 0.0, 0
#     model.train()
#     for x, y in train_iter:
#         model_outputs, enc_weight_output = model(x)
#         final_weight = enc_weight_output
#         l_loss = loss(model_outputs, y)
#         optimizer.zero_grad()
#         l_loss.backward()
#         optimizer.step()
#         l_sum += l_loss.item() * y.shape[0]
#         n += y.shape[0]
#     scheduler.step()
#     val_loss = evaluate_model_hyper_graph_transformer_weight(model, loss, val_iter)
#     if val_loss < min_val_loss:
#         min_val_loss = val_loss
#         torch.save(model.state_dict(), save_path)
#     print("epoch", epoch, ", train loss:", l_sum / n, ", validation loss:", val_loss)


best_model = HyperGraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
                                    n_node=n_node, hyper_graph_lap=hyper_graph_lap, line_graph_lap=line_graph_lap,
                                    output_size=target_len,drop=dropout_rate).to(device)
best_model.load_state_dict(torch.load(save_path))
MAE, MAPE,RMSE = evaluate_metric_hyper_graph_transformer(best_model, test_iter, scaler)
print("HyperGraph_pemsbay_30min")
print("test metric:", "\nMAE:", MAE,"\nMAPE:", MAPE, ", RMSE:", RMSE)
'''torch.Size([325, 325])
epoch 1 , train loss: 0.38639461797477603 , validation loss: 0.19880591983652493
epoch 2 , train loss: 0.15044936142035398 , validation loss: 0.1679662770020366
epoch 3 , train loss: 0.13926943589388602 , validation loss: 0.1599000961702147
epoch 4 , train loss: 0.1342812736914254 , validation loss: 0.15680580245133002
epoch 5 , train loss: 0.13159921854630782 , validation loss: 0.1523879029093208
epoch 6 , train loss: 0.1297281288505645 , validation loss: 0.15012834631001926
epoch 7 , train loss: 0.1287010208154353 , validation loss: 0.14912565486431675
epoch 8 , train loss: 0.1280923337787986 , validation loss: 0.14881685331754343
epoch 9 , train loss: 0.1274314413720437 , validation loss: 0.14723081403386354
epoch 10 , train loss: 0.12666301622247259 , validation loss: 0.14830623957627226
epoch 11 , train loss: 0.1259341942865282 , validation loss: 0.14556165214247072
epoch 12 , train loss: 0.12544037580988582 , validation loss: 0.14658198090590033
epoch 13 , train loss: 0.12489376387723823 , validation loss: 0.14473035184180005
epoch 14 , train loss: 0.12443575637726974 , validation loss: 0.14371416912383467
epoch 15 , train loss: 0.12423824889024622 , validation loss: 0.14423329730300813
epoch 16 , train loss: 0.12353887762142193 , validation loss: 0.14325812869635257
epoch 17 , train loss: 0.12305092108664642 , validation loss: 0.1427370978777415
epoch 18 , train loss: 0.12288348518815484 , validation loss: 0.14295275836358937
epoch 19 , train loss: 0.1227378589978002 , validation loss: 0.14296837799503007
epoch 20 , train loss: 0.12225514774196622 , validation loss: 0.14427885201192414
epoch 21 , train loss: 0.12170562344479235 , validation loss: 0.14092915128715996
epoch 22 , train loss: 0.12141093625471064 , validation loss: 0.1411100517536413
epoch 23 , train loss: 0.121261884193784 , validation loss: 0.14085225322956507
epoch 24 , train loss: 0.12097534663629243 , validation loss: 0.14107790239993107
epoch 25 , train loss: 0.12077387897419439 , validation loss: 0.14211237228085313
epoch 26 , train loss: 0.12035987522188077 , validation loss: 0.14074481170161254
epoch 27 , train loss: 0.1202298237513527 , validation loss: 0.140393807629852
epoch 28 , train loss: 0.12003671550514548 , validation loss: 0.14072390894153913
epoch 29 , train loss: 0.11990730162540844 , validation loss: 0.14008117962250496
epoch 30 , train loss: 0.11970242009916865 , validation loss: 0.14136978149877344
epoch 31 , train loss: 0.11933765043349374 , validation loss: 0.1401197726401598
epoch 32 , train loss: 0.11917386684957636 , validation loss: 0.14004974472342274
epoch 33 , train loss: 0.11911299273314899 , validation loss: 0.14017177687722357
epoch 34 , train loss: 0.11906627466819304 , validation loss: 0.14015030803690107
epoch 35 , train loss: 0.11886592810480816 , validation loss: 0.13988718662609007
epoch 36 , train loss: 0.11861265034463987 , validation loss: 0.14041001535382627
epoch 37 , train loss: 0.11855323916363897 , validation loss: 0.13988988322785068
epoch 38 , train loss: 0.11842820074939849 , validation loss: 0.1411606390600598
epoch 39 , train loss: 0.11842684652879706 , validation loss: 0.14077840866889862
epoch 40 , train loss: 0.11833124453299361 , validation loss: 0.1396674373054233
epoch 41 , train loss: 0.1181247141631672 , validation loss: 0.13993373869903267
epoch 42 , train loss: 0.1180547299544692 , validation loss: 0.13948016322392437
epoch 43 , train loss: 0.11794915102108076 , validation loss: 0.13974531575579097
epoch 44 , train loss: 0.11791925413644623 , validation loss: 0.13958184505460616
epoch 45 , train loss: 0.11782065393276599 , validation loss: 0.14000803125118627
epoch 46 , train loss: 0.117708405802227 , validation loss: 0.13981602257870318
epoch 47 , train loss: 0.11767065755243016 , validation loss: 0.13969107271002068
epoch 48 , train loss: 0.11764045615741858 , validation loss: 0.1402990190534146
epoch 49 , train loss: 0.11759479624835552 , validation loss: 0.13981268589805912
epoch 50 , train loss: 0.11754983987938279 , validation loss: 0.14021793429573368
HyperGraph_pemsbay_20min
test metric: 
MAE: 1.5701916975535426 
MAPE: 0.037322496193026806 , RMSE: 2.9363796815687415'''