import random
import torch
import torch.nn as nn
import numpy as np
from sklearn.preprocessing import StandardScaler
from model_hypergraph_lstm import HyperGraphTransformerWeight
from utils2 import construct_pemsbay_hyper_graph_laplacian, construct_pemsbay_line_graph_from_hyper_graph
from utils import load_data, data_transform, evaluate_model_hyper_graph_transformer_weight, evaluate_metric_hyper_graph_transformer
from torch.utils.data import DataLoader, TensorDataset
torch.manual_seed(2333)
torch.cuda.manual_seed(2333)
np.random.seed(2333)
random.seed(2333)
torch.backends.cudnn.deterministic = True
device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
data_path = "./data/pemsbay/Speed_325.csv"
save_path = "./pretrain_model/pemsbay/model_pemsbay_HyperGCN_9_v1.pt"
day_slot = 288
n_train, n_val, n_test = 35, 5, 5
batch_size = 50
epochs = 50
lr = 1e-3
n_his = 12
n_pred = 2
n_cluster = 128
output_dim = 8
input_feature = 1
embedding_size = 64
conv_output = 8
n_node = 325
# hyper_graph_lap_np = np.load("./data/adj_text_format/hyper_graph_lap_pemsbay.npy")
# hyper_graph_lap = torch.Tensor(hyper_graph_lap_np).to(device)
# line_graph_lap_np = np.load("./data/adj_text_format/line_graph_lap_pemsbay.npy")
# line_graph_lap = torch.Tensor(line_graph_lap_np).to(device)
hyper_graph_lap = construct_pemsbay_hyper_graph_laplacian(n_clusters=n_cluster)
line_graph_lap = construct_pemsbay_line_graph_from_hyper_graph(n_clusters=n_cluster)
hidden_size = conv_output * n_node
filter_size = 512
dropout_rate = 0
n_layers = 6
target_len = 2
train, val, test = load_data(data_path, n_train * day_slot, n_val * day_slot)  # 2017年3月6日到4月30日 ,除去周末，40天
scaler = StandardScaler()
train = scaler.fit_transform(train)
val = scaler.transform(val)
test = scaler.transform(test)
x_train, y_train = data_transform(train, n_his, n_pred, day_slot, device)
x_val, y_val = data_transform(val, n_his, n_pred, day_slot, device)
x_test, y_test = data_transform(test, n_his, n_pred, day_slot, device)
train_data = TensorDataset(x_train, y_train)
train_iter = DataLoader(train_data, batch_size, shuffle=True)
val_data = TensorDataset(x_val, y_val)
val_iter = DataLoader(val_data, batch_size)
test_data = TensorDataset(x_test, y_test)
test_iter = DataLoader(test_data, batch_size)

loss = nn.MSELoss()
# model = HyperGraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
#                                     n_node=n_node, hyper_graph_lap=hyper_graph_lap, line_graph_lap=line_graph_lap,
#                                     output_size=target_len,drop=dropout_rate).to(device)
# optimizer = torch.optim.Adam(model.parameters(), lr=lr)
# scheduler = torch.optim.lr_scheduler.StepLR(optimizer, step_size=5, gamma=0.7)
# min_val_loss = np.inf
# for epoch in range(1, epochs + 1):
#     l_sum, n = 0.0, 0
#     model.train()
#     for x, y in train_iter:
#         #print(x.shape)
#         model_outputs, enc_weight_output = model(x)
#         final_weight = enc_weight_output
#         l_loss = loss(model_outputs, y)
#         optimizer.zero_grad()
#         l_loss.backward()
#         optimizer.step()
#         l_sum += l_loss.item() * y.shape[0]
#         n += y.shape[0]
#     scheduler.step()
#     val_loss = evaluate_model_hyper_graph_transformer_weight(model, loss, val_iter)
#     if val_loss < min_val_loss:
#         min_val_loss = val_loss
#         torch.save(model.state_dict(), save_path)
#     print("epoch", epoch, ", train loss:", l_sum / n, ", validation loss:", val_loss)

best_model = HyperGraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
                                    n_node=n_node, hyper_graph_lap=hyper_graph_lap, line_graph_lap=line_graph_lap,
                                    output_size=target_len,drop=dropout_rate).to(device)
best_model.load_state_dict(torch.load(save_path))
MAE, MAPE,RMSE = evaluate_metric_hyper_graph_transformer(best_model, test_iter, scaler)
print("HyperGraph_pemsbay_10min")
print("test metric:", "\nMAE:", MAE,"\nMAPE:", MAPE, ", RMSE:", RMSE)
'''epoch 1 , train loss: 0.3212345783419387 , validation loss: 0.12501019881128464
epoch 2 , train loss: 0.0904452407528837 , validation loss: 0.09809475515115361
epoch 3 , train loss: 0.0814412899386573 , validation loss: 0.09175389677777997
epoch 4 , train loss: 0.0787608368928258 , validation loss: 0.0907797102076038
epoch 5 , train loss: 0.07780789649010399 , validation loss: 0.08922900966007628
epoch 6 , train loss: 0.07642503534884203 , validation loss: 0.08711743814386685
epoch 7 , train loss: 0.07603019582232143 , validation loss: 0.08684813661113748
epoch 8 , train loss: 0.07532334018068045 , validation loss: 0.08602027829903405
epoch 9 , train loss: 0.07497765031352056 , validation loss: 0.08740488472564856
epoch 10 , train loss: 0.0745865175221338 , validation loss: 0.08548474445630953
epoch 11 , train loss: 0.07400032611964469 , validation loss: 0.08480639922447494
epoch 12 , train loss: 0.07379655346279584 , validation loss: 0.08457764934734618
epoch 13 , train loss: 0.07356914257561185 , validation loss: 0.08442630327305162
epoch 14 , train loss: 0.07335434132650256 , validation loss: 0.08461636465410625
epoch 15 , train loss: 0.07317818162448714 , validation loss: 0.08386275838215779
epoch 16 , train loss: 0.07276307350370868 , validation loss: 0.08351986094236921
epoch 17 , train loss: 0.07260312053094918 , validation loss: 0.0836083121400953
epoch 18 , train loss: 0.07259077894702767 , validation loss: 0.08323773547163563
epoch 19 , train loss: 0.07240010962032395 , validation loss: 0.08299751231986999
epoch 20 , train loss: 0.07234258497599026 , validation loss: 0.08288813820218531
epoch 21 , train loss: 0.07202837172568946 , validation loss: 0.08313277877528455
epoch 22 , train loss: 0.07191646417131624 , validation loss: 0.08270900941019589
epoch 23 , train loss: 0.07190632576654338 , validation loss: 0.08253586423904785
epoch 24 , train loss: 0.07173340422935795 , validation loss: 0.08262784707188837
epoch 25 , train loss: 0.07168485822110639 , validation loss: 0.08261890297541706
epoch 26 , train loss: 0.07140112048977704 , validation loss: 0.08236461620273047
epoch 27 , train loss: 0.07136722193393658 , validation loss: 0.08234185301869851
epoch 28 , train loss: 0.07128263985934018 , validation loss: 0.08228677405129328
epoch 29 , train loss: 0.07123559825268914 , validation loss: 0.0823029803896785
epoch 30 , train loss: 0.07119523789014524 , validation loss: 0.0822371608428766
epoch 31 , train loss: 0.07102333327539706 , validation loss: 0.08233438401729658
epoch 32 , train loss: 0.07095365506025114 , validation loss: 0.0821053269145905
epoch 33 , train loss: 0.07092879455548397 , validation loss: 0.08211218778962577
epoch 34 , train loss: 0.07090625434399528 , validation loss: 0.08212412627994504
epoch 35 , train loss: 0.07083607569423422 , validation loss: 0.08198759647192212
epoch 36 , train loss: 0.07073717398312429 , validation loss: 0.08194468487374859
epoch 37 , train loss: 0.07072315983858764 , validation loss: 0.08185012998241832
epoch 38 , train loss: 0.07064851354482346 , validation loss: 0.08187581446203575
epoch 39 , train loss: 0.07061110561282996 , validation loss: 0.0818162261814235
epoch 40 , train loss: 0.0706153196676191 , validation loss: 0.08178693194430021
epoch 41 , train loss: 0.07049736312402007 , validation loss: 0.0817908941408701
epoch 42 , train loss: 0.07046719303410595 , validation loss: 0.08170840198565334
epoch 43 , train loss: 0.07046339108053294 , validation loss: 0.0818226518111847
epoch 44 , train loss: 0.0704248432844786 , validation loss: 0.08160915469710901
epoch 45 , train loss: 0.07041115368812768 , validation loss: 0.08184627144742701
epoch 46 , train loss: 0.070336187118135 , validation loss: 0.08179993981092955
epoch 47 , train loss: 0.07031534332209982 , validation loss: 0.08167613594843623
epoch 48 , train loss: 0.07031562614378352 , validation loss: 0.08163366071286655
epoch 49 , train loss: 0.07029045664262998 , validation loss: 0.08169392829777164
epoch 50 , train loss: 0.07027840579963399 , validation loss: 0.081644286703187
HyperGraph_pemsbay_10min
test metric: 
MAE: 1.2175777863262949 
MAPE: 0.027314477606851885 , RMSE: 2.191103622548178'''