import random
import torch
import torch.nn as nn
import numpy as np
from sklearn.preprocessing import StandardScaler
from model_hyper_graph_transformer import HyperGraphTransformer
from Rebuild_HyperGraph import construct_pemsbay_hyper_graph_laplacian, construct_pemsbay_line_graph_from_hyper_graph
from utils import load_data, data_transform, evaluate_model_hyper_graph_transformer, evaluate_metric_hyper_graph_transformer
from torch.utils.data import DataLoader, TensorDataset
torch.manual_seed(2333)
torch.cuda.manual_seed(2333)
np.random.seed(2333)
random.seed(2333)
torch.backends.cudnn.deterministic = True
device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
data_path = "./data/pemsbay/Speed_325.csv"
# save_path = "./pretrain_model/pemsbay/model_pemsbay_HyperGCN_test.pt"
day_slot = 288
n_train, n_val, n_test = 35, 5, 5
batch_size = 50
epochs = 50
lr = 1e-3
n_his = 12
n_pred = 3
n_cluster = 64
output_dim = 8
input_feature = 1
embedding_size = 64
conv_output = 8
n_node = 325
print("main")
hyper_graph_lap_np = np.load("./data/adj_text_format/hyper_graph_lap_pemsbay.npy")
hyper_graph_lap = torch.Tensor(hyper_graph_lap_np).to(device)
print(hyper_graph_lap)
line_graph_lap_np = np.load("./data/adj_text_format/line_graph_lap_pemsbay.npy")
line_graph_lap = torch.Tensor(line_graph_lap_np).to(device)
print(line_graph_lap)
hidden_size = conv_output * n_node
filter_size = 512
dropout_rate = 0
n_layers = 6
target_len = n_pred
train, val, test = load_data(data_path, n_train * day_slot, n_val * day_slot)  # 2017年3月6日到4月30日 ,除去周末，40天
scaler = StandardScaler()
train = scaler.fit_transform(train)
val = scaler.transform(val)
test = scaler.transform(test)
x_train, y_train = data_transform(train, n_his, n_pred, day_slot, device)
x_val, y_val = data_transform(val, n_his, n_pred, day_slot, device)
x_test, y_test = data_transform(test, n_his, n_pred, day_slot, device)
train_data = TensorDataset(x_train, y_train)
train_iter = DataLoader(train_data, batch_size, shuffle=True)
val_data = TensorDataset(x_val, y_val)
val_iter = DataLoader(val_data, batch_size)
test_data = TensorDataset(x_test, y_test)
test_iter = DataLoader(test_data, batch_size)


loss = nn.MSELoss()
model = HyperGraphTransformer(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
                              n_node=n_node, hyper_graph_lap=hyper_graph_lap, line_graph_lap=line_graph_lap,
                              hidden_size=hidden_size, filter_size=filter_size, dropout_rate=dropout_rate,
                              n_layers=n_layers, target_len=target_len).to(device)
optimizer = torch.optim.Adam(model.parameters(), lr=lr)
scheduler = torch.optim.lr_scheduler.StepLR(optimizer, step_size=5, gamma=0.7)
min_val_loss = np.inf
for epoch in range(1, epochs + 1):
    l_sum, n = 0.0, 0
    model.train()
    for x, y in train_iter:
        targets = y
        model_outputs = model(x, y)
        l_loss = loss(model_outputs,targets)
        optimizer.zero_grad()
        l_loss.backward()
        optimizer.step()
        l_sum += l_loss.item() * y.shape[0]
        n += y.shape[0]
    scheduler.step()
    val_loss = evaluate_model_hyper_graph_transformer(model, loss, val_iter)
    if val_loss < min_val_loss:
        min_val_loss = val_loss
        #torch.save(model.state_dict(), save_path)
    print("epoch", epoch, ", train loss:", l_sum / n, ", validation loss:", val_loss)


# best_model = HyperGraphTransformer(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
#                               n_node=n_node, hyper_graph_lap=hyper_graph_lap, line_graph_lap=line_graph_lap,
#                               hidden_size=hidden_size, filter_size=filter_size, dropout_rate=dropout_rate,
#                               n_layers=n_layers, target_len=target_len).to(device)
# best_model.load_state_dict(torch.load(save_path))
# MAE, MAPE,RMSE = evaluate_metric_hyper_graph_transformer(best_model, test_iter, scaler)
# print("HyperGraph_pemsbay_15min")
# print("test metric:", "\nMAE:", MAE,"\nMAPE:", MAPE, ", RMSE:", RMSE)
'''epoch 1 , train loss: 0.5949271507428187 , validation loss: 0.5228515272392263
epoch 2 , train loss: 0.3548273053505791 , validation loss: 0.44529074218456133
epoch 3 , train loss: 0.28137849556162475 , validation loss: 0.33772597015115485
epoch 4 , train loss: 0.20322187779982195 , validation loss: 0.2534888470015457
epoch 5 , train loss: 0.13837957130581324 , validation loss: 0.19573327598317788
epoch 6 , train loss: 0.09413129745233338 , validation loss: 0.149844444493939
epoch 7 , train loss: 0.07168282097871984 , validation loss: 0.12432319657500202
epoch 8 , train loss: 0.05793653395474361 , validation loss: 0.10577465925502676
epoch 9 , train loss: 0.04834484270343855 , validation loss: 0.09586114052536564
epoch 10 , train loss: 0.04537583980967832 , validation loss: 0.09017390741785408
epoch 11 , train loss: 0.035826802747829754 , validation loss: 0.07843482695139034
epoch 12 , train loss: 0.03153731918478746 , validation loss: 0.07493192519131792
epoch 13 , train loss: 0.028713583587284937 , validation loss: 0.06949480374138138
epoch 14 , train loss: 0.025936091938630806 , validation loss: 0.06509313078580922
epoch 15 , train loss: 0.023728428279805595 , validation loss: 0.058140158943121294
epoch 16 , train loss: 0.02039502434954489 , validation loss: 0.053730924694244335
epoch 17 , train loss: 0.019091253959421094 , validation loss: 0.05103353906941204
epoch 18 , train loss: 0.01784136744166646 , validation loss: 0.04741425609837101
epoch 19 , train loss: 0.01687831593846962 , validation loss: 0.04401561595738431
epoch 20 , train loss: 0.015861257291749517 , validation loss: 0.04067051024912692
epoch 21 , train loss: 0.014279699028363402 , validation loss: 0.03713589595898294
epoch 22 , train loss: 0.013711372147814717 , validation loss: 0.035931875233904696
epoch 23 , train loss: 0.01336200138376524 , validation loss: 0.034786518841010874
epoch 24 , train loss: 0.013001923014768147 , validation loss: 0.03412509470222278
epoch 25 , train loss: 0.012711804827993084 , validation loss: 0.033950835615011554
epoch 26 , train loss: 0.011715854393856879 , validation loss: 0.03199917673483631
epoch 27 , train loss: 0.01154582715333457 , validation loss: 0.03210560039008648
epoch 28 , train loss: 0.011345237026154033 , validation loss: 0.03162334834634391
epoch 29 , train loss: 0.011118189470772036 , validation loss: 0.030061016975571276
epoch 30 , train loss: 0.010899141980968611 , validation loss: 0.030484109495479784
epoch 31 , train loss: 0.010311757238466637 , validation loss: 0.0297876483616761
epoch 32 , train loss: 0.010111642096140388 , validation loss: 0.029228474644269455
epoch 33 , train loss: 0.010010121481982163 , validation loss: 0.029320787248687058
epoch 34 , train loss: 0.009848217417135307 , validation loss: 0.029100989716538073
epoch 35 , train loss: 0.009775915139407943 , validation loss: 0.029502284714570212
epoch 36 , train loss: 0.009318779665531707 , validation loss: 0.029244290684171493
epoch 37 , train loss: 0.009166850955489653 , validation loss: 0.028648121621400187
epoch 38 , train loss: 0.009091801146445071 , validation loss: 0.028630321688184853
epoch 39 , train loss: 0.009008607508692755 , validation loss: 0.028585540384557555'''