import random
import torch
import torch.nn as nn
import numpy as np
from sklearn.preprocessing import StandardScaler
from model_hypergraph_lstm import HyperGraphTransformerEncoder
from utils2 import construct_pemsbay_hyper_graph_laplacian, construct_pemsbay_line_graph_from_hyper_graph
from utils import load_data, data_transform, evaluate_model_hyper_graph_transformer_weight, evaluate_metric_hyper_graph_transformer
from torch.utils.data import DataLoader, TensorDataset
torch.manual_seed(2333)
torch.cuda.manual_seed(2333)
np.random.seed(2333)
random.seed(2333)
torch.backends.cudnn.deterministic = True
device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
data_path = "./data/pemsbay/Speed_325.csv"
save_path = "./pretrain_model/pemsbay/model_pemsbay_HyperGCN_enc_v1.pt"
day_slot = 288
n_train, n_val, n_test = 35, 5, 5
batch_size = 50
epochs = 50
lr = 1e-3
n_his = 12
n_pred = 3
n_cluster = 64
input_feature = 1
embedding_size = 32
conv_output = 4
n_node = 325
hyper_graph_lap_np = np.load("./data/adj_text_format/hyper_graph_lap_pemsbay.npy")
hyper_graph_lap = torch.Tensor(hyper_graph_lap_np).to(device)
line_graph_lap_np = np.load("./data/adj_text_format/line_graph_lap_pemsbay.npy")
line_graph_lap = torch.Tensor(line_graph_lap_np).to(device)
# hyper_graph_lap = construct_pemsbay_hyper_graph_laplacian(n_clusters=n_cluster)
# line_graph_lap = construct_pemsbay_line_graph_from_hyper_graph(n_clusters=n_cluster)
hidden_size = conv_output * n_node
filter_size = 512
dropout_rate = 0.2
n_layers = 4
target_len = 3
train, val, test = load_data(data_path, n_train * day_slot, n_val * day_slot)  # 2017年3月6日到4月30日 ,除去周末，40天
scaler = StandardScaler()
train = scaler.fit_transform(train)
val = scaler.transform(val)
test = scaler.transform(test)
x_train, y_train = data_transform(train, n_his, n_pred, day_slot, device)
x_val, y_val = data_transform(val, n_his, n_pred, day_slot, device)
x_test, y_test = data_transform(test, n_his, n_pred, day_slot, device)
train_data = TensorDataset(x_train, y_train)
train_iter = DataLoader(train_data, batch_size, shuffle=True)
val_data = TensorDataset(x_val, y_val)
val_iter = DataLoader(val_data, batch_size)
test_data = TensorDataset(x_test, y_test)
test_iter = DataLoader(test_data, batch_size)

loss = nn.MSELoss()
model = HyperGraphTransformerEncoder(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
                                    n_node=n_node, hyper_graph_lap=hyper_graph_lap, line_graph_lap=line_graph_lap,
                                    output_size=target_len, hidden_size_enc=hidden_size, filter_size=filter_size,
                                     dropout_rate = dropout_rate, n_layers=n_layers).to(device)
optimizer = torch.optim.Adam(model.parameters(), lr=lr)
scheduler = torch.optim.lr_scheduler.StepLR(optimizer, step_size=5, gamma=0.7)
min_val_loss = np.inf
final_weight = 0
for epoch in range(1, epochs + 1):
    l_sum, n = 0.0, 0
    model.train()
    for x, y in train_iter:
        #print(x.shape)
        model_outputs, enc_weight_output = model(x)
        final_weight = enc_weight_output
        l_loss = loss(model_outputs, y)
        optimizer.zero_grad()
        l_loss.backward()
        optimizer.step()
        l_sum += l_loss.item() * y.shape[0]
        n += y.shape[0]
    scheduler.step()
    val_loss = evaluate_model_hyper_graph_transformer_weight(model, loss, val_iter)
    if val_loss < min_val_loss:
        min_val_loss = val_loss
        torch.save(model.state_dict(), save_path)
    print("epoch", epoch, ", train loss:", l_sum / n, ", validation loss:", val_loss)


# best_model = HyperGraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
#                               n_node=n_node, hyper_graph_lap=hyper_graph_lap, line_graph_lap=line_graph_lap,
#                              output_size=target_len).to(device)
# best_model.load_state_dict(torch.load(save_path))
# MAE, MAPE,RMSE = evaluate_metric_hyper_graph_transformer(best_model, test_iter, scaler)
# print("HyperGraph_pemsbay_15min")
# print("test metric:", "\nMAE:", MAE,"\nMAPE:", MAPE, ", RMSE:", RMSE)
'''epoch 1 , train loss: 0.41425456969402863 , validation loss: 0.21209556258935605
epoch 2 , train loss: 0.13658793085069804 , validation loss: 0.14270422496795154
epoch 3 , train loss: 0.11664827224861171 , validation loss: 0.1369806158194331
epoch 4 , train loss: 0.11212469493679268 , validation loss: 0.1271408435336616
epoch 5 , train loss: 0.11003424399827622 , validation loss: 0.1251391622216835
epoch 6 , train loss: 0.1074318936522375 , validation loss: 0.126067910289934
epoch 7 , train loss: 0.10656902303606453 , validation loss: 0.12156787330581242
epoch 8 , train loss: 0.10524282466009617 , validation loss: 0.12188337736801297
epoch 9 , train loss: 0.10443419017398697 , validation loss: 0.1214316040649045
epoch 10 , train loss: 0.10362500215535651 , validation loss: 0.11848087636044459
epoch 11 , train loss: 0.10201235635961049 , validation loss: 0.11773461474885805
epoch 12 , train loss: 0.10021735586819563 , validation loss: 0.11510592776354163
epoch 13 , train loss: 0.09934063441667501 , validation loss: 0.11452116922621795
epoch 14 , train loss: 0.09893313158376578 , validation loss: 0.11501058993828614
epoch 15 , train loss: 0.09825208456054228 , validation loss: 0.11358730319103219
epoch 16 , train loss: 0.0975794787016246 , validation loss: 0.11328034630258747
epoch 17 , train loss: 0.09719237278246254 , validation loss: 0.11301133389144989
epoch 18 , train loss: 0.09695364475546556 , validation loss: 0.11263130874891432
epoch 19 , train loss: 0.0965947751980964 , validation loss: 0.1125481059432558
epoch 20 , train loss: 0.0962718134895943 , validation loss: 0.1116958888181249
epoch 21 , train loss: 0.09556592113742104 , validation loss: 0.11185162279413413
epoch 22 , train loss: 0.09553426006174938 , validation loss: 0.11122646621499788
epoch 23 , train loss: 0.0950842297724318 , validation loss: 0.11128564998022367
epoch 24 , train loss: 0.09483411174707994 , validation loss: 0.11184630917546125
epoch 25 , train loss: 0.09459647644210471 , validation loss: 0.11178350838781227
epoch 26 , train loss: 0.09426608503352231 , validation loss: 0.11074753624232718
epoch 27 , train loss: 0.09388281522210858 , validation loss: 0.11062038955116747
epoch 28 , train loss: 0.0936423495516976 , validation loss: 0.11058205273935805
epoch 29 , train loss: 0.09338928114095607 , validation loss: 0.11034639223439396
epoch 30 , train loss: 0.0932252369531561 , validation loss: 0.11028562352938831
epoch 31 , train loss: 0.09292728606032642 , validation loss: 0.11036121373178752
epoch 32 , train loss: 0.09271380416718109 , validation loss: 0.11010942158412365
epoch 33 , train loss: 0.0926043660450446 , validation loss: 0.11029908037889694
epoch 34 , train loss: 0.09236629722373213 , validation loss: 0.11036897983673966
epoch 35 , train loss: 0.09219327520736194 , validation loss: 0.11013823403191052
epoch 36 , train loss: 0.09194430160133801 , validation loss: 0.10991302768527415
epoch 37 , train loss: 0.09177922716612985 , validation loss: 0.11028975005904151
epoch 38 , train loss: 0.0916218956498163 , validation loss: 0.11021694453012783
epoch 39 , train loss: 0.09156632170473405 , validation loss: 0.1100251654559351
epoch 40 , train loss: 0.09144534306197194 , validation loss: 0.11090760485302285
epoch 41 , train loss: 0.09122127175109544 , validation loss: 0.11049755384319732
epoch 42 , train loss: 0.09110308491651184 , validation loss: 0.11009115384828538
epoch 43 , train loss: 0.0909775238701851 , validation loss: 0.11005389758692119
epoch 44 , train loss: 0.09091908395585745 , validation loss: 0.10985208614676112
epoch 45 , train loss: 0.09085441741711792 , validation loss: 0.11062521536243641
epoch 46 , train loss: 0.09066927192630071 , validation loss: 0.10978123710435235
epoch 47 , train loss: 0.0905578109467533 , validation loss: 0.10988723630637179
epoch 48 , train loss: 0.09052462111152941 , validation loss: 0.10990127314975201
epoch 49 , train loss: 0.09044804183816811 , validation loss: 0.10987960663405115
epoch 50 , train loss: 0.09039676527570158 , validation loss: 0.1098387759548445
HyperGraph_pemsbay_15min
test metric: 
MAE: 1.4355189374852617 
MAPE: 0.033769472866882075 , RMSE: 2.611429851043004'''