import random
import torch
import torch.nn as nn
import numpy as np
from sklearn.preprocessing import StandardScaler
from utils import load_data, data_transform, evaluate_model_hyper_graph_transformer_weight, evaluate_metric_hyper_graph_transformer
from utils2 import construct_pemsbay_hyper_graph_laplacian, construct_pemsbay_line_graph_from_hyper_graph, construct_pemsbay_graph_normalized_adj
from torch.utils.data import DataLoader, TensorDataset
from model_hypergraph_lstm import GraphTransformerWeight
torch.manual_seed(2333)
torch.cuda.manual_seed(2333)
np.random.seed(2333)
random.seed(2333)
torch.backends.cudnn.deterministic = True
device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
data_path = "./data/pemsbay/Speed_325.csv"
save_path_raw = "./pretrain_model/pemsbay/model_pemsbay_GCN_pre"
day_slot = 288
n_train, n_val, n_test = 35, 5, 5
batch_size = 50
epochs = 50
lr = 1e-3
n_his = 12
n_preds = [2, 3, 4]
n_cluster = 128
output_dim = 8
input_feature = 1
embedding_size = 64
conv_output = 8  # 1 for Transformer only
n_node = 325
hyper_graph_lap = construct_pemsbay_hyper_graph_laplacian(n_clusters=n_cluster)
line_graph_lap = construct_pemsbay_line_graph_from_hyper_graph(n_clusters=n_cluster)
graph_lap = construct_pemsbay_graph_normalized_adj()
hidden_size = conv_output * n_node
filter_size = 512
dropout_rate = 0
n_layers = 6

for n_pred in n_preds:
    save_path = save_path_raw + str(n_pred) + ".pt"
    print(save_path)
    train, val, test = load_data(data_path, n_train * day_slot, n_val * day_slot)  # 2012年4月2日到5月31日,除去周末，44天
    scaler = StandardScaler()
    train = scaler.fit_transform(train)
    val = scaler.transform(val)
    test = scaler.transform(test)
    x_train, y_train = data_transform(train, n_his, n_pred, day_slot, device)
    x_val, y_val = data_transform(val, n_his, n_pred, day_slot, device)
    x_test, y_test = data_transform(test, n_his, n_pred, day_slot, device)
    train_data = TensorDataset(x_train, y_train)
    train_iter = DataLoader(train_data, batch_size, shuffle=True)
    val_data = TensorDataset(x_val, y_val)
    val_iter = DataLoader(val_data, batch_size)
    test_data = TensorDataset(x_test, y_test)
    test_iter = DataLoader(test_data, batch_size)

    loss = nn.MSELoss()
    model = GraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
                                  graph_lap= graph_lap, n_node=n_node,output_size=n_pred).to(device)
    # optimizer = torch.optim.Adam(model.parameters(), lr=lr)
    # scheduler = torch.optim.lr_scheduler.StepLR(optimizer, step_size=5, gamma=0.7)
    # min_val_loss = np.inf
    # for epoch in range(1, epochs + 1):
    #     l_sum, n = 0.0, 0
    #     model.train()
    #     for x, y in train_iter:
    #         model_outputs, enc_weight_output = model(x)
    #         final_weight = enc_weight_output
    #         l_loss = loss(model_outputs, y)
    #         optimizer.zero_grad()
    #         l_loss.backward()
    #         optimizer.step()
    #         l_sum += l_loss.item() * y.shape[0]
    #         n += y.shape[0]
    #     scheduler.step()
    #     val_loss = evaluate_model_hyper_graph_transformer_weight(model, loss, val_iter)
    #     if val_loss < min_val_loss:
    #         min_val_loss = val_loss
    #         torch.save(model.state_dict(), save_path)
    #     print("epoch", epoch, ", train loss:", l_sum / n, ", validation loss:", val_loss)
    best_model = GraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
                                  graph_lap= graph_lap, n_node=n_node,output_size=n_pred).to(device)
    best_model.load_state_dict(torch.load(save_path))
    MAE, MAPE,RMSE = evaluate_metric_hyper_graph_transformer(best_model, test_iter, scaler)
    print("Graph_pemsbay_15min")
    print("test metric:", "\nMAE:", MAE,"\nMAPE:", MAPE, ", RMSE:", RMSE)


# best_model = GraphTransformer(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
#                               graph_lap= graph_lap,
#                               hidden_size=hidden_size, filter_size=filter_size, dropout_rate=dropout_rate,
#                               n_layers=n_layers, target_len=target_len).to(device)
# best_model.load_state_dict(torch.load(save_path))
# MAE, MAPE,RMSE = evaluate_metric_hyper_graph_transformer(best_model, test_iter, scaler)
# print("Graph_pemsbay_15min")
# print("test metric:", "\nMAE:", MAE,"\nMAPE:", MAPE, ", RMSE:", RMSE)
'''./pretrain_model/pemsbay/model_pemsbay_GCN_pre2.pt
epoch 1 , train loss: 0.47606966906824083 , validation loss: 0.37365405237779625
epoch 2 , train loss: 0.24169483084322196 , validation loss: 0.2576080550850925
epoch 3 , train loss: 0.174060800140482 , validation loss: 0.20140086291701453
epoch 4 , train loss: 0.14495003801427728 , validation loss: 0.17576973364157064
epoch 5 , train loss: 0.1295508630622029 , validation loss: 0.1521194535746852
epoch 6 , train loss: 0.11907582043430129 , validation loss: 0.14004367695430392
epoch 7 , train loss: 0.11173542994659914 , validation loss: 0.1303807664627115
epoch 8 , train loss: 0.10537693834811955 , validation loss: 0.12155938047733394
epoch 9 , train loss: 0.09950455261199345 , validation loss: 0.1183198574398358
epoch 10 , train loss: 0.09537340659360982 , validation loss: 0.11001394854139224
epoch 11 , train loss: 0.09236823398561725 , validation loss: 0.10795211819776399
epoch 12 , train loss: 0.09089400125356034 , validation loss: 0.10754222477291132
epoch 13 , train loss: 0.08945890086562007 , validation loss: 0.10488041770654434
epoch 14 , train loss: 0.08826851453892458 , validation loss: 0.10376077412533095
epoch 15 , train loss: 0.08754410372253343 , validation loss: 0.1035459643595021
epoch 16 , train loss: 0.08623867037614442 , validation loss: 0.10247229298352634
epoch 17 , train loss: 0.08572900791913413 , validation loss: 0.10150904576655072
epoch 18 , train loss: 0.0851007123026584 , validation loss: 0.10136540842011879
epoch 19 , train loss: 0.08466647583557221 , validation loss: 0.10048477397648631
epoch 20 , train loss: 0.0842539613214581 , validation loss: 0.10192505957610078
epoch 21 , train loss: 0.08360517536043384 , validation loss: 0.09990166438461093
epoch 22 , train loss: 0.0832027266889705 , validation loss: 0.09988116980520283
epoch 23 , train loss: 0.08294917153665138 , validation loss: 0.09965194699729332
epoch 24 , train loss: 0.08259335340573157 , validation loss: 0.09946730369688002
epoch 25 , train loss: 0.08231833370511207 , validation loss: 0.09866921480950488
epoch 26 , train loss: 0.08193615543599024 , validation loss: 0.09888477903580788
epoch 27 , train loss: 0.08174666221595693 , validation loss: 0.09848443520178302
epoch 28 , train loss: 0.08148433160406995 , validation loss: 0.0986028242587644
epoch 29 , train loss: 0.08125930168967015 , validation loss: 0.09855526619902391
epoch 30 , train loss: 0.08105509540638535 , validation loss: 0.09812172676454871
epoch 31 , train loss: 0.08079927558729406 , validation loss: 0.09840458824592967
epoch 32 , train loss: 0.08060257699419529 , validation loss: 0.09838757811535151
epoch 33 , train loss: 0.08045993416865636 , validation loss: 0.09781576736278215
epoch 34 , train loss: 0.08033636582646386 , validation loss: 0.0979514847614763
epoch 35 , train loss: 0.08018567594817679 , validation loss: 0.09778496205255406
epoch 36 , train loss: 0.07996496401885742 , validation loss: 0.09776304168990181
epoch 37 , train loss: 0.07984147388245032 , validation loss: 0.09760714408235613
epoch 38 , train loss: 0.07973439671474149 , validation loss: 0.09749975307899875
epoch 39 , train loss: 0.07966198138207399 , validation loss: 0.09758122627141756
epoch 40 , train loss: 0.07953904860778081 , validation loss: 0.09724775468278335
epoch 41 , train loss: 0.0794082253126632 , validation loss: 0.0974243033928364
epoch 42 , train loss: 0.07933087951239269 , validation loss: 0.09725181819493244
epoch 43 , train loss: 0.07926131203237899 , validation loss: 0.09680919460665595
epoch 44 , train loss: 0.07920376295042313 , validation loss: 0.09716437431564048
epoch 45 , train loss: 0.07911802561244692 , validation loss: 0.09713139773627058
epoch 46 , train loss: 0.07899051760000188 , validation loss: 0.09708382265809996
epoch 47 , train loss: 0.078931137656806 , validation loss: 0.09694001400872831
epoch 48 , train loss: 0.07890390942682357 , validation loss: 0.09702356669847259
epoch 49 , train loss: 0.07885184747777486 , validation loss: 0.09696669378553599
epoch 50 , train loss: 0.07879435114952023 , validation loss: 0.09705322043578775
./pretrain_model/pemsbay/model_pemsbay_GCN_pre3.pt
epoch 1 , train loss: 0.47002548070927375 , validation loss: 0.391774735797429
epoch 2 , train loss: 0.2625543737507472 , validation loss: 0.28636986145029364
epoch 3 , train loss: 0.2012923683911695 , validation loss: 0.23580684173110283
epoch 4 , train loss: 0.172714494310879 , validation loss: 0.2145140821974971
epoch 5 , train loss: 0.15529938215673328 , validation loss: 0.18064071065456777
epoch 6 , train loss: 0.14314616729434362 , validation loss: 0.16797072879671265
epoch 7 , train loss: 0.1363574177158722 , validation loss: 0.1601856030262908
epoch 8 , train loss: 0.13171870328397067 , validation loss: 0.15348326880991084
epoch 9 , train loss: 0.12790479206969188 , validation loss: 0.1490352359104839
epoch 10 , train loss: 0.12490037205274515 , validation loss: 0.14554725618776165
epoch 11 , train loss: 0.12228854191486133 , validation loss: 0.14318615245457134
epoch 12 , train loss: 0.12086038668943193 , validation loss: 0.14127005702234557
epoch 13 , train loss: 0.11947140708009683 , validation loss: 0.1401833077733298
epoch 14 , train loss: 0.11827370851445362 , validation loss: 0.13801070903455098
epoch 15 , train loss: 0.11698795745358408 , validation loss: 0.13742200830862344
epoch 16 , train loss: 0.11592477957386814 , validation loss: 0.13728093149547166
epoch 17 , train loss: 0.11522603567781067 , validation loss: 0.13518910990290092
epoch 18 , train loss: 0.11459531052030815 , validation loss: 0.13434325848538875
epoch 19 , train loss: 0.11377565336269846 , validation loss: 0.13396933306160783
epoch 20 , train loss: 0.1130885608388085 , validation loss: 0.1336573515301931
epoch 21 , train loss: 0.11235003808276488 , validation loss: 0.13204357021620433
epoch 22 , train loss: 0.11204987357430558 , validation loss: 0.13210523526373275
epoch 23 , train loss: 0.11145812647452558 , validation loss: 0.13107969449510692
epoch 24 , train loss: 0.11116592482109446 , validation loss: 0.1322481603674066
epoch 25 , train loss: 0.11076200396656422 , validation loss: 0.13071358597767463
epoch 26 , train loss: 0.11019111006188954 , validation loss: 0.13063670476622533
epoch 27 , train loss: 0.10988349732760302 , validation loss: 0.13028045336740976
epoch 28 , train loss: 0.10962990046785157 , validation loss: 0.13095512349542537
epoch 29 , train loss: 0.10938700285317213 , validation loss: 0.13021973044658655
epoch 30 , train loss: 0.10912078827750041 , validation loss: 0.13008108253830364
epoch 31 , train loss: 0.1088638642808293 , validation loss: 0.1292511612433516
epoch 32 , train loss: 0.10857939056617472 , validation loss: 0.13000945977745848
epoch 33 , train loss: 0.10837417875245087 , validation loss: 0.13026482409447163
epoch 34 , train loss: 0.10830465719261397 , validation loss: 0.12902565554400336
epoch 35 , train loss: 0.10814646407592715 , validation loss: 0.12908588672233792
epoch 36 , train loss: 0.10784259018109528 , validation loss: 0.12865976172386842
epoch 37 , train loss: 0.10764311750964609 , validation loss: 0.12923779535634708
epoch 38 , train loss: 0.10751437952936824 , validation loss: 0.1291305981746081
epoch 39 , train loss: 0.10743487523158049 , validation loss: 0.12907392235016438
epoch 40 , train loss: 0.10724996395571997 , validation loss: 0.128538413248279
epoch 41 , train loss: 0.107053508050183 , validation loss: 0.12832065895812503
epoch 42 , train loss: 0.10699337524770641 , validation loss: 0.12874043172291885
epoch 43 , train loss: 0.10689999899797129 , validation loss: 0.12865256761608204
epoch 44 , train loss: 0.10677686189347477 , validation loss: 0.128490154684793
epoch 45 , train loss: 0.10667959473702188 , validation loss: 0.12880440779458058
epoch 46 , train loss: 0.10660324909304955 , validation loss: 0.12837613582645443
epoch 47 , train loss: 0.10651464215244734 , validation loss: 0.12865216474755592
epoch 48 , train loss: 0.10644057762233046 , validation loss: 0.1285137621566133
epoch 49 , train loss: 0.10637952650279273 , validation loss: 0.1284241342252168
epoch 50 , train loss: 0.10631897455774407 , validation loss: 0.12776503115109514
./pretrain_model/pemsbay/model_pemsbay_GCN_pre4.pt
epoch 1 , train loss: 0.5188301393682854 , validation loss: 0.4397878342608304
epoch 2 , train loss: 0.30754305912214636 , validation loss: 0.3438141640997387
epoch 3 , train loss: 0.24419138741529225 , validation loss: 0.2927343659989425
epoch 4 , train loss: 0.21006234394760992 , validation loss: 0.253156712012637
epoch 5 , train loss: 0.1836443447224273 , validation loss: 0.21213733584536157
epoch 6 , train loss: 0.16889042669610035 , validation loss: 0.200348327209943
epoch 7 , train loss: 0.16207175557968787 , validation loss: 0.191653118973554
epoch 8 , train loss: 0.15735603062956177 , validation loss: 0.18473274829647127
epoch 9 , train loss: 0.15416679547978843 , validation loss: 0.17975525835718362
epoch 10 , train loss: 0.1512998128738842 , validation loss: 0.17697374922225206
epoch 11 , train loss: 0.1485411380399883 , validation loss: 0.17264928312765482
epoch 12 , train loss: 0.14705866554862795 , validation loss: 0.1718478208962803
epoch 13 , train loss: 0.14544397975103915 , validation loss: 0.17488341111311187
epoch 14 , train loss: 0.14435607275409784 , validation loss: 0.16919520063849022
epoch 15 , train loss: 0.14302191613264006 , validation loss: 0.1679181244408273
epoch 16 , train loss: 0.14197388995344506 , validation loss: 0.16599470650474682
epoch 17 , train loss: 0.14086440780367898 , validation loss: 0.16626211086237008
epoch 18 , train loss: 0.1403729933831716 , validation loss: 0.16418747773971934
epoch 19 , train loss: 0.1395534737798319 , validation loss: 0.16344796113428886
epoch 20 , train loss: 0.1392270529364007 , validation loss: 0.1623911488476523
epoch 21 , train loss: 0.13810774356576347 , validation loss: 0.16238891392456914
epoch 22 , train loss: 0.13762888185470504 , validation loss: 0.16195398060675845
epoch 23 , train loss: 0.13729842301664882 , validation loss: 0.16338500673298184
epoch 24 , train loss: 0.1368745386732733 , validation loss: 0.1610229727229377
epoch 25 , train loss: 0.13630125449287805 , validation loss: 0.16131324662579616
epoch 26 , train loss: 0.13581768545407025 , validation loss: 0.16049925415268282
epoch 27 , train loss: 0.13540819531674816 , validation loss: 0.16018818324957435
epoch 28 , train loss: 0.13524768813320676 , validation loss: 0.1599415181075885
epoch 29 , train loss: 0.13488662253126754 , validation loss: 0.1596777648468243
epoch 30 , train loss: 0.13471975802727867 , validation loss: 0.16013763198394562
epoch 31 , train loss: 0.13418131942555506 , validation loss: 0.15984681633413506
epoch 32 , train loss: 0.1339776879566238 , validation loss: 0.15885777929387504
epoch 33 , train loss: 0.1337845926855023 , validation loss: 0.15902914334469417
epoch 34 , train loss: 0.13362375179114608 , validation loss: 0.16218656689621438
epoch 35 , train loss: 0.13356559448583352 , validation loss: 0.16006947058901158
epoch 36 , train loss: 0.13315587498600123 , validation loss: 0.15831796177133392
epoch 37 , train loss: 0.13298433169766488 , validation loss: 0.1583679516860468
epoch 38 , train loss: 0.1328711443691707 , validation loss: 0.15870285555618868
epoch 39 , train loss: 0.13265868111433005 , validation loss: 0.15963468709811918
epoch 40 , train loss: 0.13264675120295477 , validation loss: 0.158491280415806
epoch 41 , train loss: 0.1323504176586933 , validation loss: 0.15827269938756383
epoch 42 , train loss: 0.13222939020753405 , validation loss: 0.15846563665568553
epoch 43 , train loss: 0.13213523227510632 , validation loss: 0.15828615180690667
epoch 44 , train loss: 0.1320901064059076 , validation loss: 0.15783155733345575
epoch 45 , train loss: 0.13196647970085054 , validation loss: 0.1579584091000443
epoch 46 , train loss: 0.1318065135945876 , validation loss: 0.15868543940030277
epoch 47 , train loss: 0.13174087497163495 , validation loss: 0.15791352361654107
epoch 48 , train loss: 0.13166421458794442 , validation loss: 0.15811589093623946
epoch 49 , train loss: 0.13161083503063042 , validation loss: 0.15809512985150825
epoch 50 , train loss: 0.13153418677009684 , validation loss: 0.1579381650795265
./pretrain_model/pemsbay/model_pemsbay_GCN_pre2.pt
Graph_pemsbay_15min
test metric: 
MAE: 1.3157789520491807 
MAPE: 0.03166001638469648 , RMSE: 2.4067298694874935
./pretrain_model/pemsbay/model_pemsbay_GCN_pre3.pt
Graph_pemsbay_15min
test metric: 
MAE: 1.5079879532769553 
MAPE: 0.03712828264698619 , RMSE: 2.8167158767403313
./pretrain_model/pemsbay/model_pemsbay_GCN_pre4.pt
Graph_pemsbay_15min
test metric: 
MAE: 1.6815073758955041 
MAPE: 0.042075420178724225 , RMSE: 3.1764977953618323'''