import random
import torch
import torch.nn as nn
import numpy as np
from sklearn.preprocessing import StandardScaler
from utils import load_data, data_transform, evaluate_model_hyper_graph_transformer_weight, evaluate_metric_hyper_graph_transformer
from utils2 import construct_pemsbay_hyper_graph_laplacian, construct_pemsbay_line_graph_from_hyper_graph, construct_pemsbay_graph_normalized_adj
from torch.utils.data import DataLoader, TensorDataset
from model_hypergraph_lstm import HGraphTransformerWeight
torch.manual_seed(2333)
torch.cuda.manual_seed(2333)
np.random.seed(2333)
random.seed(2333)
torch.backends.cudnn.deterministic = True
device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
data_path = "./data/pemsbay/Speed_325.csv"
save_path_raw = "./pretrain_model/pemsbay/model_pemsbay_HGCN_pre"
day_slot = 288
n_train, n_val, n_test = 35, 5, 5
batch_size = 50
epochs = 50
lr = 1e-3
n_his = 12
n_preds = [2, 3, 4]
n_cluster = 128
output_dim = 8
input_feature = 1
embedding_size = 64
conv_output = 8  # 1 for Transformer only
n_node = 325
hyper_graph_lap = construct_pemsbay_hyper_graph_laplacian(n_clusters=n_cluster)
print(hyper_graph_lap.shape)
line_graph_lap = construct_pemsbay_line_graph_from_hyper_graph(n_clusters=n_cluster)
graph_lap = construct_pemsbay_graph_normalized_adj()
hidden_size = conv_output * n_node
filter_size = 512
dropout_rate = 0
n_layers = 6

for n_pred in n_preds:
    save_path = save_path_raw + str(n_pred) + ".pt"
    print(save_path)
    train, val, test = load_data(data_path, n_train * day_slot, n_val * day_slot)  # 2012年4月2日到5月31日,除去周末，44天
    scaler = StandardScaler()
    train = scaler.fit_transform(train)
    val = scaler.transform(val)
    test = scaler.transform(test)
    x_train, y_train = data_transform(train, n_his, n_pred, day_slot, device)
    x_val, y_val = data_transform(val, n_his, n_pred, day_slot, device)
    x_test, y_test = data_transform(test, n_his, n_pred, day_slot, device)
    train_data = TensorDataset(x_train, y_train)
    train_iter = DataLoader(train_data, batch_size, shuffle=True)
    val_data = TensorDataset(x_val, y_val)
    val_iter = DataLoader(val_data, batch_size)
    test_data = TensorDataset(x_test, y_test)
    test_iter = DataLoader(test_data, batch_size)

    loss = nn.MSELoss()
    model = HGraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size,
                                    conv_output_size=conv_output, hyper_graph_lap=hyper_graph_lap,
                                    n_node=n_node,output_size=n_pred).to(device)
    # optimizer = torch.optim.Adam(model.parameters(), lr=lr)
    # scheduler = torch.optim.lr_scheduler.StepLR(optimizer, step_size=5, gamma=0.7)
    # min_val_loss = np.inf
    # for epoch in range(1, epochs + 1):
    #     l_sum, n = 0.0, 0
    #     model.train()
    #     for x, y in train_iter:
    #         model_outputs, enc_weight_output = model(x)
    #         final_weight = enc_weight_output
    #         l_loss = loss(model_outputs, y)
    #         optimizer.zero_grad()
    #         l_loss.backward()
    #         optimizer.step()
    #         l_sum += l_loss.item() * y.shape[0]
    #         n += y.shape[0]
    #     scheduler.step()
    #     val_loss = evaluate_model_hyper_graph_transformer_weight(model, loss, val_iter)
    #     if val_loss < min_val_loss:
    #         min_val_loss = val_loss
    #         torch.save(model.state_dict(), save_path)
    #     print("epoch", epoch, ", train loss:", l_sum / n, ", validation loss:", val_loss)
    best_model = HGraphTransformerWeight(input_feature=input_feature, embedding_size=embedding_size,
                                    conv_output_size=conv_output, hyper_graph_lap=hyper_graph_lap,
                                    n_node=n_node,output_size=n_pred).to(device)
    best_model.load_state_dict(torch.load(save_path))
    MAE, MAPE,RMSE = evaluate_metric_hyper_graph_transformer(best_model, test_iter, scaler)
    print("HGraph_pemsbay_15min")
    print("test metric:", "\nMAE:", MAE,"\nMAPE:", MAPE, ", RMSE:", RMSE)



# best_model = HGraphTransformer(input_feature=input_feature, embedding_size=embedding_size, conv_output_size=conv_output,
#                               hyper_graph_lap=hyper_graph_lap,
#                               hidden_size=hidden_size, filter_size=filter_size, dropout_rate=dropout_rate,
#                               n_layers=n_layers, target_len=target_len).to(device)
# best_model.load_state_dict(torch.load(save_path))
# MAE, MAPE,RMSE = evaluate_metric_hyper_graph_transformer(best_model, test_iter, scaler)
# print("HGraph_pemsbay_15min")
# print("test metric:", "\nMAE:", MAE,"\nMAPE:", MAPE, ", RMSE:", RMSE)
'''torch.Size([325, 325])
./pretrain_model/pemsbay/model_pemsbay_HGCN_pre2.pt
epoch 1 , train loss: 0.47056598896292334 , validation loss: 0.3956174092347486
epoch 2 , train loss: 0.2737361094596033 , validation loss: 0.31618652818480086
epoch 3 , train loss: 0.2285642985431546 , validation loss: 0.27782501341551424
epoch 4 , train loss: 0.20282964697474562 , validation loss: 0.25269701080471074
epoch 5 , train loss: 0.18405656669828313 , validation loss: 0.2303967032573218
epoch 6 , train loss: 0.1704283378143169 , validation loss: 0.21901553911704097
epoch 7 , train loss: 0.15753656241386688 , validation loss: 0.19792867756997173
epoch 8 , train loss: 0.1440925489524881 , validation loss: 0.17719950201009765
epoch 9 , train loss: 0.1327315685847989 , validation loss: 0.16707486393281312
epoch 10 , train loss: 0.12438482191682732 , validation loss: 0.15571323365768103
epoch 11 , train loss: 0.11851645715961603 , validation loss: 0.14768931535101668
epoch 12 , train loss: 0.11421172504628058 , validation loss: 0.14576785979001375
epoch 13 , train loss: 0.1104222711701609 , validation loss: 0.13654709196246476
epoch 14 , train loss: 0.10746029975504805 , validation loss: 0.13278786671200077
epoch 15 , train loss: 0.10435332660181987 , validation loss: 0.13132713493155906
epoch 16 , train loss: 0.10152929131342694 , validation loss: 0.12936001934267108
epoch 17 , train loss: 0.10002065780537557 , validation loss: 0.1243594790555419
epoch 18 , train loss: 0.09828202114836408 , validation loss: 0.12343808105390229
epoch 19 , train loss: 0.09683723061288355 , validation loss: 0.12010354985479152
epoch 20 , train loss: 0.09542126097762384 , validation loss: 0.11998860850062414
epoch 21 , train loss: 0.09398950193156787 , validation loss: 0.1182643933126433
epoch 22 , train loss: 0.09331975848328158 , validation loss: 0.11632038269517983
epoch 23 , train loss: 0.09221130636415002 , validation loss: 0.11660218642162445
epoch 24 , train loss: 0.0913350906229906 , validation loss: 0.1148454564860678
epoch 25 , train loss: 0.09060556927449545 , validation loss: 0.1135898928672264
epoch 26 , train loss: 0.08979130495069255 , validation loss: 0.11298874294737199
epoch 27 , train loss: 0.08930884162818584 , validation loss: 0.11233811067033678
epoch 28 , train loss: 0.08883216704573808 , validation loss: 0.11239130038165968
epoch 29 , train loss: 0.08822832833832134 , validation loss: 0.11217948787978348
epoch 30 , train loss: 0.08779515963819236 , validation loss: 0.11180220649732006
epoch 31 , train loss: 0.08713634415716555 , validation loss: 0.11059647087054042
epoch 32 , train loss: 0.0867854638682151 , validation loss: 0.10991769873681263
epoch 33 , train loss: 0.0864610453133911 , validation loss: 0.11001684543344188
epoch 34 , train loss: 0.08630499246319509 , validation loss: 0.10929584466709771
epoch 35 , train loss: 0.08594132412885175 , validation loss: 0.10858075961666207
epoch 36 , train loss: 0.08547516525752336 , validation loss: 0.10849210168778357
epoch 37 , train loss: 0.08526814028977028 , validation loss: 0.10913137506506325
epoch 38 , train loss: 0.08503672565111987 , validation loss: 0.10817599294608315
epoch 39 , train loss: 0.08490466314476312 , validation loss: 0.10885393263499629
epoch 40 , train loss: 0.0847086036434178 , validation loss: 0.10809048252597603
epoch 41 , train loss: 0.08436599403293826 , validation loss: 0.1081866452767693
epoch 42 , train loss: 0.08423175613616544 , validation loss: 0.10725387658191948
epoch 43 , train loss: 0.08414350966733533 , validation loss: 0.10681756049519023
epoch 44 , train loss: 0.083981123360789 , validation loss: 0.10688058513282532
epoch 45 , train loss: 0.08379312627203131 , validation loss: 0.10667603136015141
epoch 46 , train loss: 0.08359782722525753 , validation loss: 0.10671299005708056
epoch 47 , train loss: 0.08348348948172539 , validation loss: 0.10703100800477909
epoch 48 , train loss: 0.08341410274123948 , validation loss: 0.10688028963226633
epoch 49 , train loss: 0.08333965532649984 , validation loss: 0.1066170027926782
epoch 50 , train loss: 0.08319460647645076 , validation loss: 0.1064226893762731
./pretrain_model/pemsbay/model_pemsbay_HGCN_pre3.pt
epoch 1 , train loss: 0.4846184789744216 , validation loss: 0.4221068623062668
epoch 2 , train loss: 0.2936787301442075 , validation loss: 0.33791884054323607
epoch 3 , train loss: 0.24570184954449642 , validation loss: 0.3119148642422187
epoch 4 , train loss: 0.21965575708648324 , validation loss: 0.27387271287252407
epoch 5 , train loss: 0.2026344292707859 , validation loss: 0.2519691117149515
epoch 6 , train loss: 0.19073761337479792 , validation loss: 0.24247641828563157
epoch 7 , train loss: 0.18304023641105313 , validation loss: 0.23141181625169732
epoch 8 , train loss: 0.1752658207485715 , validation loss: 0.22235242252182802
epoch 9 , train loss: 0.1681700156757902 , validation loss: 0.21875772829657025
epoch 10 , train loss: 0.1620216658483456 , validation loss: 0.2020973764402995
epoch 11 , train loss: 0.15640871832114156 , validation loss: 0.20132487373012573
epoch 12 , train loss: 0.15198346845379967 , validation loss: 0.19190234188057725
epoch 13 , train loss: 0.14802102128509903 , validation loss: 0.18554177956923382
epoch 14 , train loss: 0.1439318223181017 , validation loss: 0.17871148707267603
epoch 15 , train loss: 0.1399025719434336 , validation loss: 0.17375670604965776
epoch 16 , train loss: 0.13562975326984505 , validation loss: 0.17229443430942648
epoch 17 , train loss: 0.13310913439692454 , validation loss: 0.16559698439839268
epoch 18 , train loss: 0.13055295958983926 , validation loss: 0.16282379610718026
epoch 19 , train loss: 0.12783985751314902 , validation loss: 0.16129755512294722
epoch 20 , train loss: 0.12580917817894913 , validation loss: 0.15791468626898356
epoch 21 , train loss: 0.12378337970127147 , validation loss: 0.15397951955622452
epoch 22 , train loss: 0.12231810889387149 , validation loss: 0.15297817456603702
epoch 23 , train loss: 0.12092877954395563 , validation loss: 0.15121541541009562
epoch 24 , train loss: 0.11982140557864121 , validation loss: 0.15015999147767806
epoch 25 , train loss: 0.1186582799277094 , validation loss: 0.14826189852043475
epoch 26 , train loss: 0.1174445405838865 , validation loss: 0.1502376235005781
epoch 27 , train loss: 0.11662189621112479 , validation loss: 0.1463849482767942
epoch 28 , train loss: 0.1160611238815843 , validation loss: 0.14863101096391387
epoch 29 , train loss: 0.1154347087405966 , validation loss: 0.14563654380711463
epoch 30 , train loss: 0.11461356907300312 , validation loss: 0.14400634163927367
epoch 31 , train loss: 0.11395354849027194 , validation loss: 0.1442338120214223
epoch 32 , train loss: 0.11329441891874299 , validation loss: 0.14349091247162915
epoch 33 , train loss: 0.11300183602899698 , validation loss: 0.14329221557793703
epoch 34 , train loss: 0.11256457652797336 , validation loss: 0.1419704296305843
epoch 35 , train loss: 0.11212355697721248 , validation loss: 0.1429215636189804
epoch 36 , train loss: 0.11155923955375194 , validation loss: 0.14227505054039002
epoch 37 , train loss: 0.11131439377709292 , validation loss: 0.1426500705012347
epoch 38 , train loss: 0.1110069222876736 , validation loss: 0.1416129918799845
epoch 39 , train loss: 0.11071683832649168 , validation loss: 0.14235495903268616
epoch 40 , train loss: 0.11037057649598386 , validation loss: 0.14147319881028933
epoch 41 , train loss: 0.1100657777421071 , validation loss: 0.14071089075192214
epoch 42 , train loss: 0.10986976827450486 , validation loss: 0.14187144285085398
epoch 43 , train loss: 0.10967811659235739 , validation loss: 0.14156705056788454
epoch 44 , train loss: 0.1094856634962379 , validation loss: 0.1414842951871477
epoch 45 , train loss: 0.1092698643258668 , validation loss: 0.14141994276642414
epoch 46 , train loss: 0.1090745591437394 , validation loss: 0.14050135266974056
epoch 47 , train loss: 0.10890888216709539 , validation loss: 0.14116911100077773
epoch 48 , train loss: 0.10879452348951968 , validation loss: 0.14112243123175117
epoch 49 , train loss: 0.10865949813368228 , validation loss: 0.14017145165705086
epoch 50 , train loss: 0.10854671437955726 , validation loss: 0.13975547195726293
./pretrain_model/pemsbay/model_pemsbay_HGCN_pre4.pt
epoch 1 , train loss: 0.5297150073391252 , validation loss: 0.44984624392497735
epoch 2 , train loss: 0.320849417268197 , validation loss: 0.3657426283130181
epoch 3 , train loss: 0.27023171020992226 , validation loss: 0.3204000440163006
epoch 4 , train loss: 0.24320952754804268 , validation loss: 0.297156021105239
epoch 5 , train loss: 0.22525388197010135 , validation loss: 0.2781315165196847
epoch 6 , train loss: 0.21275016492319695 , validation loss: 0.2702023309596051
epoch 7 , train loss: 0.20561300554617915 , validation loss: 0.25537506033500873
epoch 8 , train loss: 0.19837636644034784 , validation loss: 0.24624746032345038
epoch 9 , train loss: 0.19064553912638704 , validation loss: 0.2369212613479226
epoch 10 , train loss: 0.1839850215504437 , validation loss: 0.23159647334617398
epoch 11 , train loss: 0.17779364984586551 , validation loss: 0.22254649638163232
epoch 12 , train loss: 0.17395558712814904 , validation loss: 0.21604412222298078
epoch 13 , train loss: 0.16935118691876752 , validation loss: 0.20812496326917845
epoch 14 , train loss: 0.1648333873262818 , validation loss: 0.20382418593859222
epoch 15 , train loss: 0.16077298929100084 , validation loss: 0.1989457307172526
epoch 16 , train loss: 0.157341206121289 , validation loss: 0.19311995960886238
epoch 17 , train loss: 0.1548711407285488 , validation loss: 0.19335114462210276
epoch 18 , train loss: 0.1529946231957454 , validation loss: 0.186698403433587
epoch 19 , train loss: 0.15075498800952133 , validation loss: 0.18486256615141092
epoch 20 , train loss: 0.14952627421157952 , validation loss: 0.18260476714808954
epoch 21 , train loss: 0.14736836821123542 , validation loss: 0.1811380266704057
epoch 22 , train loss: 0.14619380820655228 , validation loss: 0.18012258969089484
epoch 23 , train loss: 0.14529967979034875 , validation loss: 0.18073139886250936
epoch 24 , train loss: 0.14443981975803813 , validation loss: 0.17760144459857893
epoch 25 , train loss: 0.1430007395837886 , validation loss: 0.17741451206152226
epoch 26 , train loss: 0.14219690448531588 , validation loss: 0.17615465099671265
epoch 27 , train loss: 0.14145195432865643 , validation loss: 0.174020190037937
epoch 28 , train loss: 0.14102910658767595 , validation loss: 0.1741765560770207
epoch 29 , train loss: 0.14018981335530237 , validation loss: 0.1732484802593058
epoch 30 , train loss: 0.13980409538235647 , validation loss: 0.17349847106275643
epoch 31 , train loss: 0.13894904152948556 , validation loss: 0.17262993721772113
epoch 32 , train loss: 0.13857493450370623 , validation loss: 0.17189417306794955
epoch 33 , train loss: 0.13815978178559668 , validation loss: 0.17257499730944556
epoch 34 , train loss: 0.13767812777405733 , validation loss: 0.17308583690266188
epoch 35 , train loss: 0.13749844294798208 , validation loss: 0.17284257770620476
epoch 36 , train loss: 0.13697074674899526 , validation loss: 0.17081076451720617
epoch 37 , train loss: 0.13667683866114416 , validation loss: 0.17121787011335896
epoch 38 , train loss: 0.13648600133641936 , validation loss: 0.17079567696234124
epoch 39 , train loss: 0.13621812978459835 , validation loss: 0.17132563102620083
epoch 40 , train loss: 0.13598708457010678 , validation loss: 0.17124599727195636
epoch 41 , train loss: 0.13558029418905843 , validation loss: 0.17012736150471053
epoch 42 , train loss: 0.13540480154191206 , validation loss: 0.17017102100581502
epoch 43 , train loss: 0.13520423694632958 , validation loss: 0.17010645607010938
epoch 44 , train loss: 0.1351380585715848 , validation loss: 0.16918074461700608
epoch 45 , train loss: 0.13491614819979997 , validation loss: 0.16968733262873567
epoch 46 , train loss: 0.1346748458748001 , validation loss: 0.17014100491852197
epoch 47 , train loss: 0.1345580667959666 , validation loss: 0.16925099679328828
epoch 48 , train loss: 0.13444807903098674 , validation loss: 0.16913893221382983
epoch 49 , train loss: 0.13435104536908174 , validation loss: 0.1695219874979938
epoch 50 , train loss: 0.13421156626382993 , validation loss: 0.16918128764328888
torch.Size([325, 325])
./pretrain_model/pemsbay/model_pemsbay_HGCN_pre2.pt
HGraph_pemsbay_15min
test metric: 
MAE: 1.4111501908318296 
MAPE: 0.03681699023021961 , RMSE: 2.609737952898235
./pretrain_model/pemsbay/model_pemsbay_HGCN_pre3.pt
HGraph_pemsbay_15min
test metric: 
MAE: 1.5997996421715577 
MAPE: 0.04184552404656222 , RMSE: 3.005784284670241
./pretrain_model/pemsbay/model_pemsbay_HGCN_pre4.pt
HGraph_pemsbay_15min
test metric: 
MAE: 1.7636706993451845 
MAPE: 0.046352994733888424 , RMSE: 3.313060690950165'''
