import torch
import numpy as np
import pandas as pd
device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")


def data_prepare(file_path, day_slot=288):  # 适用于PEMSBAY，工作日45天，和PEMSD4， 43天
    data_seq = pd.read_csv(file_path, header=None).values.astype(float)
    data_seq_flow = np.array(data_seq)
    weekend_index = []
    for i in range(1, 9):
        weekend_index.append(7 * i - 2)
        weekend_index.append(7 * i - 1)
    day_sum = int(data_seq_flow.shape[0] / day_slot)
    data_seq_flow_weekend = pd.DataFrame()
    data_seq_flow_weekday = pd.DataFrame()
    for i in range(day_sum):
        data_seq_flow_day = data_seq_flow[day_slot * i: day_slot * (i + 1), :]
        if i in weekend_index:
            data_seq_flow_weekend = pd.concat([data_seq_flow_weekend, pd.DataFrame(data_seq_flow_day)], axis=0)
        else:
            data_seq_flow_weekday = pd.concat([data_seq_flow_weekday, pd.DataFrame(data_seq_flow_day)], axis=0)
    df = pd.DataFrame(data_seq_flow_weekday)
    return df


def data_prepare_pems08(file_path, day_slot=288):  # 2016年7月至8月的数据，共62天数据,根据日历剔除周末,44天
    data_seq = pd.read_csv(file_path, header=None).values.astype(float)
    data_seq_flow = np.array(data_seq)
    weekend_index = []
    for i in range(1, 9):
        weekend_index.append(7 * i + 1)
        weekend_index.append(7 * i + 2)
    weekend_index.append(1)  # 将第一周的数据分开存放
    weekend_index.append(2)
    day_sum = int(data_seq_flow.shape[0] / day_slot)
    data_seq_flow_weekend = pd.DataFrame()
    data_seq_flow_weekday = pd.DataFrame()
    for i in range(day_sum):
        data_seq_flow_day = data_seq_flow[day_slot * i: day_slot * (i + 1), :]
        if i in weekend_index:
            data_seq_flow_weekend = pd.concat([data_seq_flow_weekend, pd.DataFrame(data_seq_flow_day)], axis=0)
        else:
            data_seq_flow_weekday = pd.concat([data_seq_flow_weekday, pd.DataFrame(data_seq_flow_day)], axis=0)
    df = pd.DataFrame(data_seq_flow_weekday)
    return df


def data_prepare_pemsm(file_path):  # 从2012年的五月到六月工作日的数据，共44天。
    data_seq_flow_weekday = pd.read_csv(file_path, header=None).values.astype(float)
    df = pd.DataFrame(data_seq_flow_weekday)
    print(df.shape)
    return df


def load_data(file_path, len_train, len_val):
    df = data_prepare(file_path)
    train = df[: len_train]
    val = df[len_train: len_train + len_val]
    test = df[len_train + len_val:]
    test = test.replace(0, np.nan)
    return train, val, test


def load_data_pems08(file_path, len_train, len_val):
    df = data_prepare_pems08(file_path)
    train = df[: len_train]
    val = df[len_train: len_train + len_val]
    test = df[len_train + len_val:]
    test = test.replace(0, np.nan)
    return train, val, test


def load_data_pemsm(file_path, len_train, len_val):
    df = data_prepare_pemsm(file_path)
    train = df[: len_train]
    val = df[len_train: len_train + len_val]
    test = df[len_train + len_val:]
    test = test.replace(0, np.nan)
    return train, val, test


def data_transform(data, n_his, n_pred, day_slot, device):
    n_day = len(data) // day_slot
    n_route = data.shape[1]
    n_slot = day_slot - n_his - n_pred + 1
    x = np.zeros([n_day * n_slot, 1, n_his, n_route])
    y = np.zeros([n_day * n_slot,n_pred,n_route])
    for i in range(n_day):
        for j in range(n_slot):
            t = i * n_slot + j
            s = i * day_slot + j
            e = s + n_his
            x[t, :, :, :] = data[s:e].reshape(1, n_his, n_route)
            y[t, :, :] = data[e:e + n_pred].reshape(n_pred,n_route)
    return torch.Tensor(x).to(device), torch.Tensor(y).to(device)



def data_transform_trans(data, n_his, n_pred, day_slot, device):
    n_day = len(data) // day_slot
    n_route = data.shape[1]
    n_slot = day_slot - n_his - n_pred + 1
    x = np.zeros([n_day * n_slot, 1, n_his, n_route])
    y = np.zeros([n_day * n_slot,n_route])
    for i in range(n_day):
        for j in range(n_slot):
            t = i * n_slot + j
            s = i * day_slot + j
            e = s + n_his
            x[t, :, :, :] = data[s:e].reshape(1, n_his, n_route)
            y[t] = data[e + n_pred-1]
    return torch.Tensor(x).to(device), torch.Tensor(y).to(device)



def evaluate_model_hyper_graph_transformer(model, loss, data_iter):
    model.eval()
    l_sum, n = 0.0, 0
    with torch.no_grad():
        for x, y in data_iter:
            target = y
            label = y[:, -1, :]
            start_seq = torch.Tensor([1]).unsqueeze(0).unsqueeze(1).repeat(target.shape[0], 1, target.shape[2]).to(
                device)
            target_update = torch.cat((start_seq, target), 1)
            y_pred,_ = model(x,  target_update)
            l_loss = loss(y_pred[:, -1, :], label)
            l_sum += l_loss.item() * y.shape[0]
            n += y.shape[0]
        return l_sum / n


def evaluate_model_hyper_graph_transformer_weight(model, loss, data_iter):
    model.eval()
    l_sum, n = 0.0, 0
    with torch.no_grad():
        for x, y in data_iter:
            y_pred, _ = model(x)
            l_loss = loss(y_pred, y)
            l_sum += l_loss.item() * y.shape[0]
            n += y.shape[0]
        return l_sum / n



def masked_mse(preds, labels, null_val=np.nan):
    if np.isnan(null_val):
        mask = ~torch.isnan(labels)
    else:
        mask = (labels!=null_val)
    mask = mask.float()
    mask /= torch.mean((mask))
    mask = torch.where(torch.isnan(mask), torch.zeros_like(mask), mask)
    loss = (preds-labels)**2
    loss = loss * mask
    loss = torch.where(torch.isnan(loss), torch.zeros_like(loss), loss)
    return torch.mean(loss)

def masked_rmse(preds, labels, null_val=np.nan):
    return torch.sqrt(masked_mse(preds=preds, labels=labels, null_val=null_val))


def masked_mae(preds, labels, null_val=np.nan):
    if np.isnan(null_val):
        mask = ~torch.isnan(labels)
    else:
        mask = (labels!=null_val)
    mask = mask.float()
    mask /=  torch.mean((mask))
    mask = torch.where(torch.isnan(mask), torch.zeros_like(mask), mask)
    loss = torch.abs(preds-labels)
    loss = loss * mask
    loss = torch.where(torch.isnan(loss), torch.zeros_like(loss), loss)
    return torch.mean(loss)


def masked_mape(preds, labels, null_val=np.nan):
    if np.isnan(null_val):
        mask = ~torch.isnan(labels)
    else:
        mask = (labels!=null_val)
    mask = mask.float()
    mask /=  torch.mean((mask))
    mask = torch.where(torch.isnan(mask), torch.zeros_like(mask), mask)
    loss = torch.abs(preds-labels)/labels
    loss = loss * mask
    loss = torch.where(torch.isnan(loss), torch.zeros_like(loss), loss)
    return torch.mean(loss)


def metric(pred, real):
    mae = masked_mae(pred,real,0.0).item()
    mape = masked_mape(pred,real,0.0).item()
    rmse = masked_rmse(pred,real,0.0).item()
    return mae,mape,rmse


def evaluate_metric_hyper_graph_transformer(model, data_iter, scaler):
    model.eval()
    y_pred_save = []
    ground_truth = []
    with torch.no_grad():
        mae, mape, rmse = [], [], []
        for x, y in data_iter:
            y_pred, _ = model(x)
            y = scaler.inverse_transform(y.squeeze().cpu())
            ground_truth.extend(y[:, -1, 270].tolist())
            y = torch.Tensor(y)
            y_pred = scaler.inverse_transform(y_pred.cpu())
            y_pred_save.extend(y_pred[:, -1, 270].tolist())
            y_pred = torch.Tensor(y_pred)
            mae.append(metric(y_pred, y)[0])
            mape.append(metric(y_pred, y)[1])
            rmse.append(metric(y_pred, y)[2])
        MAE = np.array(mae).mean()
        MAPE = np.array(mape).mean()
        RMSE = np.array(rmse).mean()
        pd_y_pred_save = pd.DataFrame(y_pred_save)
        pd_ground_truth = pd.DataFrame(ground_truth)
        pd_y_pred_save.to_csv("./pretrain_model/prediction_epoch_50_270.csv", header=False, index=False)
        pd_ground_truth.to_csv("./pretrain_model/ground_truth_270.csv", header=False, index=False)
        return MAE, MAPE, RMSE


