import csv
import torch
import numpy as np
import pandas as pd
from sklearn.preprocessing import StandardScaler  # z-score归一化表示的包
from sklearn.metrics.pairwise import euclidean_distances
from sklearn.cluster import KMeans
device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
# device = torch.device("cpu")


def get_pemsbay_weekday_data():  # Speed_325 : 2017年3月6日到5月5日 61天，同样需要将周末过滤掉,45天, 对数据实行z-score归一化
    adj_filename = "./data/pemsbay/adj_mx_bay.npy"
    graph_signal_matrix_filename = "./data/pemsbay/Speed_325.csv"
    num_of_vertices = 325
    day_slot = 288
    distance_pattern = np.load(adj_filename)
    data_seq = pd.read_csv(graph_signal_matrix_filename, header=None).values.astype(float)
    data_seq_flow = np.array(data_seq)
    data_scaler = StandardScaler()
    data_seq_flow = data_scaler.fit_transform(data_seq_flow)
    day_sum = int(data_seq_flow.shape[0]/day_slot)
    weekend_index = []
    for i in range(1,9):
        weekend_index.append(7*i-2)
        weekend_index.append(7*i-1)
    data_seq_flow_weekend = np.zeros((day_slot,num_of_vertices))
    data_seq_flow_weekday = np.zeros((day_slot,num_of_vertices))
    countSum = 0  # 用于数据集中休息日的计数，将数据集首先进行分类，进行全局的关系发现
    countSum2 = 0  # 用于数据集中工作日的计数
    for i in range(day_sum):
        data_seq_flow_day = data_seq_flow[day_slot * i: day_slot * (i + 1), :]
        if i in weekend_index:
            data_seq_flow_weekend = data_seq_flow_weekend + np.array(data_seq_flow_day)
            countSum += 1
        else:
            data_seq_flow_weekday = data_seq_flow_weekday + np.array(data_seq_flow_day)
            countSum2 += 1
    weekend_avg = (1 / countSum) * data_seq_flow_weekend
    weekday_avg = (1 / countSum2) * data_seq_flow_weekday
    return distance_pattern, weekday_avg, weekend_avg



def construct_pemsbay_hyper_graph_from_cluster(clusters):
    # clusters 是指K-means聚成多少类，从全局构建超边
    distance, X, _ = get_pemsbay_weekday_data()
    # distance 是路网的局部连接关系
    X = X.transpose()
    X = np.array(X)
    k_means = KMeans(n_clusters=clusters, random_state=0).fit(X)
    centers = k_means.cluster_centers_
    dis = euclidean_distances(X, centers)
    _, cluster_center_dict = torch.topk(torch.Tensor(dis), 1, dim=0, largest=False)  # 取得距离簇中心点最近的节点
    cluster_center_dict = cluster_center_dict.numpy()  # 几个关键路口的交通速度分布是否具有一些相似性
    point_labels = k_means.labels_
    point_in_which_cluster = [np.where(point_labels == i)[0] for i in range(clusters)]
    return cluster_center_dict[0], point_in_which_cluster, distance


def expand_pemsbay_hyper_graph_from_adj(clusters):
    center_clusters, point_in_which_cluster, distance = construct_pemsbay_hyper_graph_from_cluster(clusters)
    #print(center_clusters)
    np_dis = np.array(distance)
    hyper_edges = []
    adj_nodes = []
    for center in center_clusters:
        one_order_adj_nodes = np.where((np_dis[center, :] != 0) & (np_dis[center, :] != 1))  # 除去自身之外的物理连接节点
        adj_nodes.append(one_order_adj_nodes[0])
    for i in range(clusters):
        hyper_edge = np.array(point_in_which_cluster[i]).tolist() + np.array(adj_nodes[i]).tolist()
        hyper_edges.append(list(set(hyper_edge)))
    return hyper_edges, np_dis.shape[0]


# 如何从超边中生成矩阵表示，进行代码实现，并进行超图卷积
def construct_pemsbay_hyper_graph_laplacian(n_clusters):
    hyper_edges, vertex_size = expand_pemsbay_hyper_graph_from_adj(n_clusters)
    correlation_matrix_H = np.zeros((vertex_size, len(hyper_edges)))
    vertex_degree_matrix_D = np.zeros(vertex_size)
    edge_degree_matrix_D = np.zeros(len(hyper_edges))
    hyper_edges_weight_matrix_W = np.ones(len(hyper_edges))  # 这里先默认所有的超边权重均为1
    identity_matrix_I = np.ones(vertex_size)
    for j in range(len(hyper_edges)):
        edge_degree_matrix_D[j] = len(hyper_edges[j])
    for node in range(vertex_size):
        for i in range(len(hyper_edges)):
            hyper_edge = hyper_edges[i]
            if node in hyper_edge:
                vertex_degree_matrix_D[node] += 1
                correlation_matrix_H[node][i] = 1
    H = np.mat(correlation_matrix_H)
    D_v_2 = np.mat(np.diag(np.power(vertex_degree_matrix_D, -0.5)))
    D_inv = np.mat(np.diag(np.power(edge_degree_matrix_D, -1)))
    W = np.mat(np.diag(hyper_edges_weight_matrix_W))
    I_m = np.mat(np.diag(identity_matrix_I))
    theta = D_v_2 * H * W * D_inv * H.transpose() * D_v_2
    Nor_hyper_graph_L = I_m - theta
    return torch.Tensor(Nor_hyper_graph_L).to(device)


# 如何生成线图，并产生线图卷积
def construct_pemsbay_line_graph_from_hyper_graph(n_clusters):
    hyper_edges, vertex_size = expand_pemsbay_hyper_graph_from_adj(n_clusters)
    weight_matrix_line_graph = np.zeros((len(hyper_edges), len(hyper_edges)))
    for i in range(len(hyper_edges)):
        temp_i_hyper_edge = hyper_edges[i]
        for j in range(len(hyper_edges)):
            # 取出超边的交集与并集部分，计算线图的其权重,如果是本身，则权重为1，因为交集和并集都是自己
            inter_section = list(set(temp_i_hyper_edge).intersection(hyper_edges[j]))
            union_section = list(set(temp_i_hyper_edge).union(hyper_edges[j]))
            if inter_section:
                weight_matrix_line_graph[i][j] = len(inter_section) / len(union_section)
    D_hat = np.sum(weight_matrix_line_graph, 1)
    A_hat = np.mat(weight_matrix_line_graph)
    D_hat_2 = np.mat(np.diag(np.power(D_hat, -0.5)))
    line_graph_nor = D_hat_2 * A_hat * D_hat_2
    return torch.Tensor(line_graph_nor).to(device)


def construct_pemsbay_graph_normalized_adj():
    A, _, _ = get_pemsbay_weekday_data()
    D = np.array(np.sum(A, axis=1)).reshape((-1,))
    D[D <= 10e-5] = 10e-5    # Prevent infs
    diag = np.reciprocal(np.sqrt(D))
    A_wave = np.multiply(np.multiply(diag.reshape((-1, 1)), A),
                         diag.reshape((1, -1)))
    return torch.Tensor(np.mat(A_wave)).to(device)


